#include <stdio.h>
#include <stdlib.h>
#include <QtGui/QWidget>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QTabWidget>
#include <QtGui/QMainWindow>
#include <QtCore/QFile>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtGui/QLabel>
#include <QtGui/QGroupBox>
#include <QtGui/QRadioButton>
#include <QtGui/QButtonGroup>
#include <QtGui/QTreeWidget>
#include <QtGui/QTreeWidgetItem>
#include <QtGui/QPushButton>
#include <QtGui/QCheckBox>
#include <QtCore/QTextStream>
#include <QtCore/QStringList>
#include "model.h"
#include "synth.h"
#include "harmonics.h"
#include "filter.h"
#include "multislider.h"
#include "modslider.h"
#include "gui.h"
#include "jackprocessor.h"

Gui::Gui(Model *p_model, QWidget *parent) : QWidget(parent) {

  QWidget *harmonicWidget[MAX_PARAMS];
  QVBoxLayout *harmonicLayout[MAX_PARAMS];
  QWidget *scaleBox[MAX_PARAMS];
  QHBoxLayout *scaleLayout[MAX_PARAMS];
  QLabel *scaleLabel[MAX_PARAMS];
  QString labels[MAX_PARAMS], groupLabels[MAX_SCENES], radioLabels[MAX_SOURCES];
  QRadioButton *radio[MAX_SCENES][MAX_SOURCES];
  QGroupBox *groupBox[MAX_SCENES];
  QHBoxLayout *radioLayout[MAX_SCENES];
  QVBoxLayout *spectrumLayout[2], *spectrumComboLayout[2];
  QHBoxLayout *spectrumHLayout[2];
  QWidget *spectrumHFrame[2], *spectrumComboFrame[2];
  QVBoxLayout *spectrumLayoutL[2], *spectrumLayoutR[2];
  QWidget *spectrumBox[2], *spectrumBoxL[2], *spectrumBoxR[2];
  QGroupBox *evenOddBox;
  QVBoxLayout *evenOddLayout;
  QLabel *spectrumLabel[8];
  int i1, i2, i3, i4;
  QVBoxLayout *filterMainLayout[2];
  QWidget *filterFrame[2], *filterHSubFrame[2], *filterVSubFrame[6];
  QWidget *filterMixFrame[2], *filterMixVSubFrame[MAX_SOURCES][3];
  QWidget *filterModFrame[2], *filterModHSubFrame[2][6], *filterModVSubFrame[2][12];
  QGroupBox *filterMixBox[MAX_SOURCES];
  QGroupBox *filterModBox[2][3];
  QString filterMixBoxLabels[MAX_SOURCES], filterMixLabels[MAX_SOURCES][9];
  QString filterModBoxLabels[2][3], filterModLabels[2][20], filterModLongLabels[2][20];
  QHBoxLayout *filterHSubFrameLayout[2];
  QVBoxLayout *filterVSubFrameLayout[6];
  QVBoxLayout *filterMixMainLayout[2];
  QHBoxLayout *filterMixHSubFrameLayout[MAX_SOURCES];
  QVBoxLayout *filterMixVSubFrameLayout[MAX_SOURCES][3];
  QVBoxLayout *filterModMainLayout[2];
  QHBoxLayout *filterModHSubFrameLayout[2][6];
  QVBoxLayout *filterModVSubFrameLayout[2][12];
  QVBoxLayout *filterModGroupBoxLayout[2][3];
  QLabel *filterCutoffLabel[4], *filterSpacingLabel[4], *filterKeyTrackingLabel[2], *filterDepthLabel[2];
  QWidget *noiseFrame;
  QVBoxLayout *noiseFrameLayout;
  QGroupBox *noiseBox[2];
  QVBoxLayout *noiseBoxLayout[2];
  QString noiseLabels[4];
  QGroupBox *compressorBox;
  QVBoxLayout *compressorLayout;
  QString compressorLabels[4];
  QGroupBox *panBox;
  QVBoxLayout *panLayout;
  QString panLabels[3];
  QWidget *voiceFrame;
  QVBoxLayout *voiceLayout;
  QGroupBox *voiceFileFrame;
  QHBoxLayout *voiceFileLayout;
  QGroupBox *voiceRangeFrame;    
  QHBoxLayout *voiceRangeLayout;
  QGroupBox *voiceTreeFrame;    
  QHBoxLayout *voiceTreeLayout;
  QStringList voiceTreeHeaders;
  QString unitLabels[MAX_PARAMS];
  
  model = p_model;
  jackProcessor = new JackProcessor(model);
  synth = new Synth(model);
  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainTab = new QTabWidget;
  mainLayout->addWidget(mainTab);
  QVBoxLayout *harmonicMainLayout = new QVBoxLayout;
  QWidget *harmonicFrame = new QWidget;
  harmonicFrame->setLayout(harmonicMainLayout);
  harmonicTab = new QTabWidget;
  harmonicTab->setTabPosition(QTabWidget::South);
  harmonicMainLayout->addWidget(harmonicTab);
  mainTab->addTab(harmonicFrame, "Harmonics");

  QVBoxLayout *oscMainLayout = new QVBoxLayout;
  QWidget *oscFrame = new QWidget;
  oscFrame->setLayout(oscMainLayout);
  mainTab->addTab(oscFrame, "Oscillators");

  voiceFileName = "<No File>";
  noiseFrameLayout = new QVBoxLayout;
  noiseFrame = new QWidget;
  noiseFrame->setLayout(noiseFrameLayout);
  mainTab->addTab(noiseFrame, "Noise");
  
  noiseLabels[0] = "Offset";
  noiseLabels[1] = "Spacing";
  noiseLabels[2] = "Even/Odd";
  noiseLabels[3] = "Bandwidth";
  noiseBox[0] = new QGroupBox("Noise Spectrum");
  noiseBox[1] = new QGroupBox("Bandpass Filter");
  for (i1 = 0; i1 < 2; i1++) {
    noiseBoxLayout[i1] = new QVBoxLayout;
    noiseBox[i1]->setLayout(noiseBoxLayout[i1]);
    noiseFrameLayout->addWidget(noiseBox[i1]);
  } 
  for (i1 = 0; i1 < 4; i1++) {
    noiseSlider[i1] = new ModSlider(model, Qt::Horizontal); 
    noiseSlider[i1]->setMinimum(0);
    noiseSlider[i1]->setMaximum(1000);
    noiseSlider[i1]->setTickInterval(1);
    QObject::connect(noiseSlider[i1], SIGNAL(valueChanged(int)), this, SLOT(setNoise(int)));
  }
  for (i1 = 0; i1 < 3; i1++) {
    noiseBoxLayout[0]->addWidget(new QLabel(noiseLabels[i1]));
    noiseBoxLayout[0]->addWidget(noiseSlider[i1]);
    noiseSlider[i1]->setValue(500);
  }
  noiseSlider[1]->setValue(0);
  noiseBoxLayout[1]->addWidget(new QLabel(noiseLabels[3]));
  noiseBoxLayout[1]->addWidget(noiseSlider[3]);
  noiseSlider[3]->setValue(500);
  noiseFrameLayout->addWidget(new QWidget); // Dummy
  noiseFrameLayout->addWidget(new QWidget); // Dummy
  noiseFrameLayout->setContentsMargins(10, 40, 10, 40); 
  
  filterMixBoxLabels[0] = "Spectral Oscillator 1";  
  filterMixBoxLabels[1] = "Spectral Oscillator 2";
  filterMixBoxLabels[2] = "Spectral Oscillator 3";
  filterMixBoxLabels[3] = "Noise";
  for (i1 = 0; i1 < 2; i1++) {
    filterMixFrame[i1] = new QWidget;
    filterMixMainLayout[i1] = new QVBoxLayout;
    filterMixFrame[i1]->setLayout(filterMixMainLayout[i1]);
  }  
  for (i1 = 0; i1 < MAX_SOURCES; i1++) {
    filterMixLabels[i1][0] = "Filter 1/2 Morph " + QString::number(i1 + 1);
    filterMixLabels[i1][1] = "Serial/Parallel " + QString::number(i1 + 1);
    filterMixLabels[i1][2] = "LFO Mode " + QString::number(i1 + 1);
    filterMixLabels[i1][3] = "Envelope Amount " + QString::number(i1 + 1);
    filterMixLabels[i1][4] = "Delay " + QString::number(i1 + 1);
    filterMixLabels[i1][5] = "LFO Retrigger " + QString::number(i1 + 1);
    filterMixLabels[i1][6] = "Attack " + QString::number(i1 + 1);
    filterMixLabels[i1][7] = "Decay " + QString::number(i1 + 1);
    filterMixLabels[i1][8] = "Hold " + QString::number(i1 + 1);
    filterMixBox[i1] = new QGroupBox(filterMixBoxLabels[i1]);
    filterMixHSubFrameLayout[i1] = new QHBoxLayout; 
    filterMixBox[i1]->setLayout(filterMixHSubFrameLayout[i1]);
    for (i2 = 0; i2 < 3; i2++) {
      filterMixVSubFrameLayout[i1][i2] = new QVBoxLayout; 
      filterMixVSubFrame[i1][i2] = new QWidget;
      filterMixVSubFrame[i1][i2]->setLayout(filterMixVSubFrameLayout[i1][i2]);
    }
    for (i2 = 0; i2 < 6; i2++) {
      filterMixSlider[i1][i2] = new ModSlider(model, Qt::Horizontal); 
      filterMixSlider[i1][i2]->setMinimum(0);
      filterMixSlider[i1][i2]->setMaximum(1000);
      filterMixSlider[i1][i2]->setTickInterval(1);
      QObject::connect(filterMixSlider[i1][i2], SIGNAL(valueChanged(int)), this, SLOT(setFilterMix(int)));
    }
    filterMixSlider[i1][0]->setValue(500);
    filterMixSlider[i1][1]->setValue(1000);
    filterMixSlider[i1][2]->setValue(100);
    filterMixSlider[i1][3]->setValue(100);
    filterMixSlider[i1][4]->setValue(100);
    filterMixSlider[i1][5]->setValue(100);
    for (i2 = 0; i2 < 3; i2++) {  
      filterMixCheckBox[i1][i2] = new QCheckBox(filterMixLabels[i1][3 * i2 + 2]);
      QObject::connect(filterMixCheckBox[i1][i2], SIGNAL(stateChanged(int)), this, SLOT(filterMixToggled(int)));
      filterMixCheckBox[i1][i2]->setChecked(false);
    }
    for (i3 = 0; i3 < 2; i3++) {
      for (i2 = 0; i2 < 3; i2++) {
        filterMixVSubFrameLayout[i1][i3]->addWidget(new QLabel(filterMixLabels[i1][3 * i2 + i3]));
        filterMixVSubFrameLayout[i1][i3]->addWidget(filterMixSlider[i1][2 * i2 + i3]);
      }
      filterMixHSubFrameLayout[i1]->addWidget(filterMixVSubFrame[i1][i3]);
    }
    for (i2 = 0; i2 < 3; i2++) {
//      filterMixVSubFrameLayout[i1][2]->addWidget(new QLabel(filterMixLabels[i1][3 * i2 + 2]));
      filterMixVSubFrameLayout[i1][2]->addWidget(filterMixCheckBox[i1][i2]);
    }
    filterMixHSubFrameLayout[i1]->addWidget(filterMixVSubFrame[i1][2]);
    filterMixMainLayout[i1 / 2]->addWidget(filterMixBox[i1]);
  }
  mainTab->addTab(filterMixFrame[0], "Mixer 1/2");
  mainTab->addTab(filterMixFrame[1], "Mixer 3/Noise");

  for (i1 = 0; i1 < 2; i1++) {
    filterMainLayout[i1] = new QVBoxLayout;
    filterFrame[i1] = new QWidget;
    filterFrame[i1]->setLayout(filterMainLayout[i1]);
    filter[i1] = new Filter(model, i1, this, 800, 450);
    filterMainLayout[i1]->addWidget(filter[i1]);
    filterHSubFrameLayout[i1] = new QHBoxLayout; 
    filterHSubFrame[i1] = new QWidget;
    filterHSubFrame[i1]->setLayout(filterHSubFrameLayout[i1]);
    filterVSubFrameLayout[3 * i1] = new QVBoxLayout; 
    filterVSubFrame[3 * i1] = new QWidget;
    filterVSubFrame[3 * i1]->setLayout(filterVSubFrameLayout[3 * i1]);
    filterVSubFrameLayout[3 * i1 + 1] = new QVBoxLayout; 
    filterVSubFrame[3 * i1 + 1] = new QWidget;
    filterVSubFrame[3 * i1 + 1]->setLayout(filterVSubFrameLayout[3 * i1 + 1]);
    filterVSubFrameLayout[3 * i1 + 2] = new QVBoxLayout; 
    filterVSubFrame[3 * i1 + 2] = new QWidget;
    filterVSubFrame[3 * i1 + 2]->setLayout(filterVSubFrameLayout[3 * i1 + 2]);
    
    for (i2 = 0; i2 < 2; i2++) {
      filterCutoff[2 * i1 + i2] = new ModSlider(model, Qt::Horizontal); 
      filterSpacing[2 * i1 + i2] = new ModSlider(model, Qt::Horizontal);
      filterCutoff[2 * i1 + i2]->setMinimum(0);
      filterCutoff[2 * i1 + i2]->setMaximum(1000);
      filterCutoff[2 * i1 + i2]->setTickInterval(1);
      filterSpacing[2 * i1 + i2]->setMinimum(0);
      filterSpacing[2 * i1 + i2]->setMaximum(1000);
      filterSpacing[2 * i1 + i2]->setTickInterval(1);
    }  
    filterCutoffLabel[2 * i1] = new QLabel("Cutoff " + QString::number(i1 + 1) + " Coarse");
    filterSpacingLabel[2 * i1] = new QLabel("Spacing " + QString::number(i1 + 1) + " Coarse");
    filterCutoffLabel[2 * i1 + 1] = new QLabel("Cutoff " + QString::number(i1 + 1) + " Fine");
    filterSpacingLabel[2 * i1 + 1] = new QLabel("Spacing " + QString::number(i1 + 1) + " Fine");
    filterVSubFrameLayout[3 * i1]->addWidget(filterCutoffLabel[2 * i1]);
    filterVSubFrameLayout[3 * i1]->addWidget(filterCutoff[2 * i1]);
    filterVSubFrameLayout[3 * i1]->addWidget(filterSpacingLabel[2 * i1]);
    filterVSubFrameLayout[3 * i1]->addWidget(filterSpacing[2 * i1]);
    filterKeyTracking[i1] = new ModSlider(model, Qt::Horizontal); 
    filterKeyTracking[i1]->setMinimum(0);
    filterKeyTracking[i1]->setMaximum(1000);
    filterKeyTracking[i1]->setTickInterval(1);
    filterKeyTrackingLabel[i1] = new QLabel("KeyTracking " + QString::number(i1 + 1));
    filterVSubFrameLayout[3 * i1]->addWidget(filterKeyTrackingLabel[i1]);
    filterVSubFrameLayout[3 * i1]->addWidget(filterKeyTracking[i1]);
    
    filterDepth[i1] = new ModSlider(model, Qt::Horizontal);
    filterDepth[i1]->setMinimum(0);
    filterDepth[i1]->setMaximum(1000);
    filterDepth[i1]->setTickInterval(1);
    filterDepthLabel[i1] = new QLabel("Mix " + QString::number(i1 + 1));
    filterVSubFrameLayout[3 * i1 + 1]->addWidget(filterCutoffLabel[2 * i1 + 1]);
    filterVSubFrameLayout[3 * i1 + 1]->addWidget(filterCutoff[2 * i1 + 1]);
    filterVSubFrameLayout[3 * i1 + 1]->addWidget(filterSpacingLabel[2 * i1 + 1]);
    filterVSubFrameLayout[3 * i1 + 1]->addWidget(filterSpacing[2 * i1 + 1]);
    filterVSubFrameLayout[3 * i1 + 1]->addWidget(filterDepthLabel[i1]);
    filterVSubFrameLayout[3 * i1 + 1]->addWidget(filterDepth[i1]);

    filterCutoffCombo[i1] = new QComboBox;
    filterCutoffCombo[i1]->addItem("Free");
    for (i2 = 0; i2 < 14; i2++) {
      filterCutoffCombo[i1]->addItem(QString::number(i2 * 12 - 108));
    }
    filterSpacingCombo[i1] = new QComboBox;
    filterSpacingCombo[i1]->addItem("Free");
    for (i2 = 1; i2 < 25; i2++) {
      filterSpacingCombo[i1]->addItem(QString::number((double)i2 * 0.5, 'f', 2));
    }
    filterTrackingCombo[i1] = new QComboBox;
    filterTrackingCombo[i1]->addItem("Free");
    for (i2 = 0; i2 < 5; i2++) {
      filterTrackingCombo[i1]->addItem(QString::number((double)i2 / 4.0, 'f', 2));
    }
    filterVSubFrameLayout[3 * i1 + 2]->addWidget(new QLabel("Cutoff"));
    filterVSubFrameLayout[3 * i1 + 2]->addWidget(filterCutoffCombo[i1]);
    filterVSubFrameLayout[3 * i1 + 2]->addWidget(new QLabel("Spacing"));
    filterVSubFrameLayout[3 * i1 + 2]->addWidget(filterSpacingCombo[i1]);
    filterVSubFrameLayout[3 * i1 + 2]->addWidget(new QLabel("Tracking"));
    filterVSubFrameLayout[3 * i1 + 2]->addWidget(filterTrackingCombo[i1]);
    
    filterHSubFrameLayout[i1]->addWidget(filterVSubFrame[3 * i1]);
    filterHSubFrameLayout[i1]->addWidget(filterVSubFrame[3 * i1 + 1]);
    filterHSubFrameLayout[i1]->addWidget(filterVSubFrame[3 * i1 + 2]);
    filterMainLayout[i1]->addWidget(filterHSubFrame[i1]);
    mainTab->addTab(filterFrame[i1], "Filter " + QString::number(i1 + 1));
    for (i2 = 0; i2 < 2; i2++) {
      QObject::connect(filterCutoff[2 * i1 + i2], SIGNAL(valueChanged(int)), this, SLOT(setFilterCutoff(int)));
      QObject::connect(filterSpacing[2 * i1 + i2], SIGNAL(valueChanged(int)), this, SLOT(setFilterSpacing(int)));
    }  
    QObject::connect(filterKeyTracking[i1], SIGNAL(valueChanged(int)), this, SLOT(setFilterKeyTracking(int)));
    QObject::connect(filterDepth[i1], SIGNAL(valueChanged(int)), this, SLOT(setFilterDepth(int)));
    QObject::connect(filterCutoffCombo[i1], SIGNAL(currentIndexChanged(int)), this, SLOT(setFilterCutoffCombo(int)));
    QObject::connect(filterSpacingCombo[i1], SIGNAL(currentIndexChanged(int)), this, SLOT(setFilterSpacingCombo(int)));
    QObject::connect(filterTrackingCombo[i1], SIGNAL(currentIndexChanged(int)), this, SLOT(setFilterTrackingCombo(int)));
    filterCutoff[2 * i1]->setValue(500);
    filterSpacing[2 * i1]->setValue(400);
    filterCutoff[2 * i1 + 1]->setValue(500);
    filterSpacing[2 * i1 + 1]->setValue(500);
    filterKeyTracking[i1]->setValue(500);
  }
  filterDepth[0]->setValue(500);
  filterDepth[1]->setValue(1000);
  for (i1 = 0; i1 < 2; i1++) {
    filterModMainLayout[i1] = new QVBoxLayout;
    filterModFrame[i1] = new QWidget;
    filterModFrame[i1]->setLayout(filterModMainLayout[i1]);
    filterModBoxLabels[i1][0] = "Envelope";  
    filterModBoxLabels[i1][1] = "Cutoff LFO";
    filterModBoxLabels[i1][2] = "Spacing LFO";
    filterModLongLabels[i1][0] = "Delay";
    filterModLongLabels[i1][1] = "Attack";
    filterModLongLabels[i1][2] = "Decay 1";
    filterModLongLabels[i1][3] = "Decay 2";
    filterModLongLabels[i1][4] = "Level";
    filterModLongLabels[i1][5] = "Release";
    filterModLongLabels[i1][6] = "Envelope Amount";
    filterModLongLabels[i1][7] = " LFO Freq";
    filterModLongLabels[i1][8] = " LFO Depth";
    filterModLongLabels[i1][9] = " LFO Delay";
    filterModLongLabels[i1][10] = "Spacing LFO Freq";
    filterModLongLabels[i1][11] = "Spacing LFO Depth";
    filterModLongLabels[i1][12] = "Spacing LFO Delay";
    filterModLongLabels[i1][13] = "Inverse Envelope";
    filterModLongLabels[i1][14] = "Fast LFO";
    filterModLongLabels[i1][15] = "Retrigger LFO";
    filterModLongLabels[i1][16] = "Inverse LFO";
    filterModLongLabels[i1][17] = "Fast Spacing LFO";
    filterModLongLabels[i1][18] = "Retrigger Spacing LFO";
    filterModLongLabels[i1][19] = "Inverse Spacing LFO";

    filterModLabels[i1][0] = "Delay";
    filterModLabels[i1][1] = "Attack";
    filterModLabels[i1][2] = "Decay 1";
    filterModLabels[i1][3] = "Decay 2";
    filterModLabels[i1][4] = "Level";
    filterModLabels[i1][5] = "Release";
    filterModLabels[i1][6] = "Envelope Amount";
    filterModLabels[i1][7] = "Frequency";
    filterModLabels[i1][8] = "Depth";
    filterModLabels[i1][9] = "Delay";
    filterModLabels[i1][10] = "Frequency";
    filterModLabels[i1][11] = "Depth";
    filterModLabels[i1][12] = "Delay";
    filterModLabels[i1][13] = "Inverse";
    filterModLabels[i1][14] = "Fast";
    filterModLabels[i1][15] = "Retrigger";
    filterModLabels[i1][16] = "Inverse";
    filterModLabels[i1][17] = "Fast ";
    filterModLabels[i1][18] = "Retrigger ";
    filterModLabels[i1][19] = "Inverse ";
    for (i2 = 0; i2 < 3; i2++) {
      filterModBox[i1][i2] = new QGroupBox(filterModBoxLabels[i1][i2]);
      filterModGroupBoxLayout[i1][i2] = new QVBoxLayout; 
      filterModBox[i1][i2]->setLayout(filterModGroupBoxLayout[i1][i2]);
      for (i4 = 0; i4 < 2; i4++) {
        filterModHSubFrameLayout[i1][2 * i2 + i4] = new QHBoxLayout;
        filterModHSubFrameLayout[i1][2 * i2 + i4]->setContentsMargins(0, 0, 0, 0);
        filterModHSubFrame[i1][2 * i2 + i4] = new QWidget;
        filterModHSubFrame[i1][2 * i2 + i4]->setLayout(filterModHSubFrameLayout[i1][2 * i2 + i4]);
        filterModGroupBoxLayout[i1][i2]->addWidget(filterModHSubFrame[i1][2 * i2 + i4]);
        for (i3 = 0; i3 < 2; i3++) {
          filterModVSubFrameLayout[i1][4 * i2 + 2 * i4 + i3] = new QVBoxLayout; 
          filterModVSubFrame[i1][4 * i2 + 2 * i4 + i3] = new QWidget;
          filterModVSubFrame[i1][4 * i2 + 2 * i4 + i3]->setLayout(filterModVSubFrameLayout[i1][4 * i2 + 2 * i4 + i3]);
          filterModHSubFrameLayout[i1][2 * i2 + i4]->addWidget(filterModVSubFrame[i1][4 * i2 + + 2 * i4 + i3]);
        }
      }    
    }
    for (i2 = 0; i2 < 13; i2++) {
      filterModSlider[i1][i2] = new ModSlider(model, Qt::Horizontal); 
      filterModSlider[i1][i2]->setMinimum(0);
      filterModSlider[i1][i2]->setMaximum(1000);
      filterModSlider[i1][i2]->setTickInterval(1);
      QObject::connect(filterModSlider[i1][i2], SIGNAL(valueChanged(int)), this, SLOT(setFilterMod(int)));
    }
    filterModSlider[i1][0]->setValue(100);
    filterModSlider[i1][1]->setValue(100);
    filterModSlider[i1][2]->setValue(100);
    filterModSlider[i1][3]->setValue(100);
    filterModSlider[i1][4]->setValue(100);
    filterModSlider[i1][5]->setValue(100);
    filterModSlider[i1][6]->setValue(100);
    filterModSlider[i1][7]->setValue(100);
    filterModSlider[i1][8]->setValue(100);
    filterModSlider[i1][9]->setValue(100);
    filterModSlider[i1][10]->setValue(100);
    filterModSlider[i1][11]->setValue(100);
    filterModSlider[i1][12]->setValue(100);

    for (i2 = 0; i2 < 7; i2++) {  
      filterModCheckBox[i1][i2] = new QCheckBox(filterModLabels[i1][13 + i2]);
      QObject::connect(filterModCheckBox[i1][i2], SIGNAL(stateChanged(int)), this, SLOT(filterModToggled(int)));
      filterModCheckBox[i1][i2]->setChecked(true);
      filterModCheckBox[i1][i2]->setChecked(false);
    }
    for (i2 = 0; i2 < 3; i2++) {
      for (i3 = 0; i3 < 2; i3++) {
        filterModVSubFrameLayout[i1][i3]->addWidget(new QLabel(filterModLabels[i1][2 * i2 + i3]));
        filterModVSubFrameLayout[i1][i3]->addWidget(filterModSlider[i1][2 * i2 + i3]);
      }
    }
    filterModVSubFrameLayout[i1][2]->addWidget(new QLabel(filterModLabels[i1][6]));
    filterModVSubFrameLayout[i1][2]->addWidget(filterModSlider[i1][6]);
    filterModVSubFrameLayout[i1][3]->addWidget(filterModCheckBox[i1][0]);
    for (i2 = 0; i2 < 2; i2++) {
      filterModVSubFrameLayout[i1][4 * i2 + 4]->addWidget(new QLabel(filterModLabels[i1][7 + 3 * i2 + 2]));
      filterModVSubFrameLayout[i1][4 * i2 + 4]->addWidget(filterModSlider[i1][7 + 3 * i2 + 2]);
      filterModVSubFrameLayout[i1][4 * i2 + 5]->addWidget(new QLabel(filterModLabels[i1][7 + 3 * i2 + 1]));
      filterModVSubFrameLayout[i1][4 * i2 + 5]->addWidget(filterModSlider[i1][7 + 3 * i2 + 1]);
      filterModVSubFrameLayout[i1][4 * i2 + 6]->addWidget(new QLabel(filterModLabels[i1][7 + 3 * i2]));
      filterModVSubFrameLayout[i1][4 * i2 + 6]->addWidget(filterModSlider[i1][7 + 3 * i2]);
      filterModVSubFrameLayout[i1][4 * i2 + 7]->addWidget(filterModCheckBox[i1][3 * i2 + 1]);
      filterModVSubFrameLayout[i1][4 * i2 + 7]->addWidget(filterModCheckBox[i1][3 * i2 + 2]);
      filterModVSubFrameLayout[i1][4 * i2 + 7]->addWidget(filterModCheckBox[i1][3 * i2 + 3]);
    }
    for (i2 = 0; i2 < 3; i2++) {
      filterModMainLayout[i1]->addWidget(filterModBox[i1][i2]);
    }  
    mainTab->addTab(filterModFrame[i1], "Filter " + QString::number(i1 + 1) + " Modulation");
  }
  QVBoxLayout *commonMainLayout = new QVBoxLayout;
  QWidget *commonFrame = new QWidget;
  commonFrame->setLayout(commonMainLayout);
  mainTab->addTab(commonFrame, "Main");

  voiceFrame = new QWidget;
  voiceLayout = new QVBoxLayout;
  voiceFrame->setLayout(voiceLayout);
  voiceFileFrame = new QGroupBox("Sound File");
  voiceFileLayout = new QHBoxLayout;
  voiceFileFrame->setLayout(voiceFileLayout);
  voiceFileLabel = new QLabel(voiceFileName);
  voiceFileLayout->addWidget(voiceFileLabel);
  voiceRangeFrame = new QGroupBox("Edit Range && MIDI Channel");
  voiceRangeLayout = new QHBoxLayout;
  voiceRangeFrame->setLayout(voiceRangeLayout);
  voiceRangeStart = new QSpinBox;
  voiceRangeStart->setRange(1, model->getPoly());
  voiceRangeEnd = new QSpinBox;
  voiceRangeEnd->setRange(1, model->getPoly());
  voiceRangeEnd->setValue(model->getPoly());
  voiceChannel = new QSpinBox;
  voiceChannel->setRange(1, 16);
  QObject::connect(voiceRangeStart, SIGNAL(valueChanged(int)), this, SLOT(voiceRangeStartChanged(int)));
  QObject::connect(voiceRangeEnd, SIGNAL(valueChanged(int)), this, SLOT(voiceRangeEndChanged(int)));
  QObject::connect(voiceChannel, SIGNAL(valueChanged(int)), this, SLOT(voiceChannelChanged(int)));
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QLabel("Start: "));  
  voiceRangeLayout->addWidget(voiceRangeStart);
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QLabel("End: "));  
  voiceRangeLayout->addWidget(voiceRangeEnd);  
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QLabel("Channel: "));  
  voiceRangeLayout->addWidget(voiceChannel);  
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeSetButton = new QPushButton("Set");
  QObject::connect(voiceRangeSetButton, SIGNAL(clicked()), this, SLOT(voiceRangeSet()));
  voiceRangeLayout->addWidget(voiceRangeSetButton);
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceRangeLayout->addWidget(new QWidget); // Dummy 
  voiceTreeFrame = new QGroupBox("Voice List");
  voiceTreeLayout = new QHBoxLayout;
  voiceTreeFrame->setLayout(voiceTreeLayout);
  voiceTreeHeaders.append("Voice");
  voiceTreeHeaders.append("Channel");
  voiceTreeHeaders.append("Sound File");
  voiceTree = new QTreeWidget();
  voiceTree->setColumnCount(3);
  QObject::connect(voiceTree, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), this, SLOT(voiceTreeChanged(QTreeWidgetItem *, QTreeWidgetItem *)));
  voiceTree->setHeaderLabels(voiceTreeHeaders);  
  voiceTreeLayout->addWidget(voiceTree);
  voiceLayout->addWidget(voiceFileFrame);
  voiceLayout->addWidget(voiceRangeFrame);
  voiceLayout->addWidget(voiceTreeFrame);
  voiceLayout->addWidget(new QWidget); // Dummy
  voiceLayout->addWidget(new QWidget); // Dummy
  mainTab->addTab(voiceFrame, "Voice Assign");

  QVBoxLayout *settingsMainLayout = new QVBoxLayout;
  QWidget *settingsFrame = new QWidget;
  settingsFrame->setLayout(settingsMainLayout);
  mainTab->addTab(settingsFrame, "MIDI");
  QWidget *treeFrame = new QWidget;
  settingsMainLayout->addWidget(treeFrame);
  QHBoxLayout *treeLayout = new QHBoxLayout;
  treeFrame->setLayout(treeLayout);
  midiControllerTree = new QTreeWidget();
  midiControllerTree->setColumnCount(1);
  midiControllerTree->setHeaderLabel("MIDI Input");
  treeLayout->addWidget(midiControllerTree);
  controlObjectTree = new QTreeWidget();
  controlObjectTree->setColumnCount(1);
  controlObjectTree->setHeaderLabel("Destination");
  treeLayout->addWidget(controlObjectTree);
  QTreeWidgetItem *commonItem = new QTreeWidgetItem(controlObjectTree);
  commonItem->setText(0, "Main");
  QWidget *buttonFrame = new QWidget;
  settingsMainLayout->addWidget(buttonFrame);  
  QHBoxLayout *buttonLayout = new QHBoxLayout;
  buttonFrame->setLayout(buttonLayout);
  QPushButton *connectButton = new QPushButton("Connect");
  QPushButton *disconnectButton = new QPushButton("Disconnect");
  buttonLayout->addWidget(connectButton); 
  buttonLayout->addWidget(disconnectButton); 
  QObject::connect(connectButton, SIGNAL(clicked()), this, SLOT(connectController()));
  QObject::connect(disconnectButton, SIGNAL(clicked()), this, SLOT(disconnectController()));
  QObject::connect(midiControllerTree, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), this,
                                       SLOT(midiCurrentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)));
  followMidi = true;
  QHBoxLayout *modSettingsLayout = new QHBoxLayout;
  QWidget *modSettingsFrame = new QWidget;
  modSettingsFrame->setLayout(modSettingsLayout);
  settingsMainLayout->addWidget(modSettingsFrame);
  QVBoxLayout *modSettingsLayoutL = new QVBoxLayout;
  QVBoxLayout *modSettingsLayoutR = new QVBoxLayout;
  QWidget *modSettingsFrameL = new QWidget;
  modSettingsFrameL->setLayout(modSettingsLayoutL);
  modSettingsLayout->addWidget(modSettingsFrameL);
  QWidget *modSettingsFrameR = new QWidget;
  modSettingsFrameR->setLayout(modSettingsLayoutR);
  modSettingsLayout->addWidget(modSettingsFrameR);
  modDepthLabel = new QLabel("Modulation Depth");
  modSettingsLayoutL->addWidget(modDepthLabel);
  modDepthSlider = new QSlider(Qt::Horizontal); 
  modSettingsLayoutL->addWidget(modDepthSlider);
  modDepthSlider->setMinimum(0);
  modDepthSlider->setMaximum(1000);
  modDepthSlider->setTickInterval(1);
  QObject::connect(modDepthSlider, SIGNAL(valueChanged(int)), this, SLOT(setModDepth(int)));
  modFullToggle = new QCheckBox("Full Range");
  modSettingsLayoutR->addWidget(modFullToggle);
  modInverseToggle = new QCheckBox("Inverse");
  QObject::connect(modFullToggle, SIGNAL(stateChanged(int)), this, SLOT(modFullToggled(int)));
  modFullToggle->setChecked(true);
  QObject::connect(modInverseToggle, SIGNAL(stateChanged(int)), this, SLOT(modInverseToggled(int)));
  modSettingsLayoutR->addWidget(modInverseToggle);
  modInverseToggle->setChecked(false);
  modFullToggle->setChecked(true);
  
  setLayout(mainLayout);

  labels[0] = "Amplitude";
  labels[1] = "Delay";
  labels[2] = "Attack";
  labels[3] = "Decay 1";
  labels[4] = "Decay 2";
  labels[5] = "Level";
  labels[6] = "Release";
  labels[7] = "Env LFO Amount";
  labels[8] = "Env LFO Delay";
  labels[9] = "Env LFO Freq";
  
  groupLabels[0] = "Scene 1";
  groupLabels[1] = "Scene 2";
  radioLabels[0] = "Spectral Oscillator 1";
  radioLabels[1] = "Spectral Oscillator 2";
  radioLabels[2] = "Spectral Oscillator 3";
  radioLabels[3] = "Noise";

  groupSelector = new QButtonGroup;
  QHBoxLayout *radioFrameLayout = new QHBoxLayout; 
  QWidget *radioFrame = new QWidget;
  harmonicMainLayout->addWidget(radioFrame);
  radioFrame->setLayout(radioFrameLayout);
  for (i1 = 0; i1 < MAX_SCENES; i1++) {
    groupBox[i1] = new QGroupBox(groupLabels[i1]);
    radioLayout[i1] = new QHBoxLayout;
    for (i2 = 0; i2 < MAX_SOURCES; i2++) {
      radio[i1][i2] = new QRadioButton(radioLabels[i2]);
      radioLayout[i1]->addWidget(radio[i1][i2]);
      groupSelector->addButton(radio[i1][i2]);
      groupSelector->setId(radio[i1][i2], i1 * MAX_SOURCES + i2);
    }  
    groupBox[i1]->setLayout(radioLayout[i1]);
    radioFrameLayout->addWidget(groupBox[i1]);
    QObject::connect(groupSelector, SIGNAL(buttonClicked(int)), this, SLOT(selectGroup(int)));
  }  
  radio[0][0]->setChecked(true);
  for (i1 = 0; i1 < MAX_PARAMS; i1++) {
    harmonicWidget[i1] = new QWidget;
    harmonicTab->addTab(harmonicWidget[i1], labels[i1]);
    harmonicLayout[i1] = new QVBoxLayout;
    harmonicWidget[i1]->setLayout(harmonicLayout[i1]);
    if (i1 == 0) {
      unitLabels[i1] = " dB";
    } else {
      unitLabels[i1] = " ";
    }
    harmonics[i1] = new Harmonics(model, unitLabels[i1], i1, this, 800, 450);
    harmonicLayout[i1]->addWidget(harmonics[i1]);
    scaleBox[i1] = new QWidget;
    harmonicLayout[i1]->addWidget(scaleBox[i1]);
    scaleLayout[i1] = new QHBoxLayout;
    scaleBox[i1]->setLayout(scaleLayout[i1]);
    scaleLabel[i1] = new QLabel("Scale  ");
    scaleLayout[i1]->addWidget(scaleLabel[i1]);
    harmonicScale[i1] = new MultiSlider(model, MAX_SCENES * MAX_SOURCES, Qt::Horizontal);
    harmonicScale[i1]->setMinimum(-1000);
    harmonicScale[i1]->setMaximum(1000);
    harmonicScale[i1]->setTickInterval(1);
    harmonicScale[i1]->writeValue(0, 0);
    scaleLayout[i1]->addWidget(harmonicScale[i1]);
    harmonicLayout[i1]->setStretchFactor(harmonics[i1], 10);
    harmonicLayout[i1]->setStretchFactor(scaleBox[i1], 1);
    QObject::connect(harmonicScale[i1], SIGNAL(valueChanged(int)), this, SLOT(reScale(int)));
    QObject::connect(harmonicTab, SIGNAL(currentChanged(int)), this, SLOT(updateTab(int)));
  }
  for (i1 = 0; i1 < MAX_SCENES; i1++) {
    for (i2 = 0; i2 < MAX_SOURCES; i2++) {
      addControlFolder(groupLabels[i1] + ": " + radioLabels[i2]);
      for (i3 = 0; i3 < MAX_PARAMS; i3++) {
        addControlObject(labels[i3]+" Scale", i2 + MAX_SOURCES * i1 + 1, i3);
      }
    }    
  }  
  addControlFolder("Oscillators");
  addControlFolder("Noise");
  for (i1 = 0; i1 < 4; i1++) {
    addControlObject("Noise " + noiseLabels[i1], MAX_SCENES * MAX_SOURCES + 2, i1);
  }

  addControlFolder("Mixer");
  for (i1 = 0; i1 < MAX_SOURCES; i1++) {
    for (i2 = 0; i2 < 9; i2++) {
      addControlObject(filterMixLabels[i1][i2], MAX_SCENES * MAX_SOURCES + 3, 9 * i1 + i2);
    }  
  }

  addControlFolder("Filter 1 & 2");
  for (i1 = 0; i1 < 2; i1++) {
    addControlObject("Filter Cutoff " + QString::number(i1 + 1) + " Coarse", MAX_SCENES * MAX_SOURCES + 4, 9 * i1);
    addControlObject("Filter Cutoff " + QString::number(i1 + 1) + " Fine", MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 1);
    addControlObject("Filter Spacing " + QString::number(i1 + 1) + " Coarse", MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 2);
    addControlObject("Filter Spacing " + QString::number(i1 + 1) + " Fine", MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 3);
    addControlObject("Filter Key Tracking " + QString::number(i1 + 1), MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 4);
    addControlObject("Filter Mix " + QString::number(i1 + 1), MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 5);
    addControlObject("Filter Cutoff Fixed Values " + QString::number(i1 + 1), MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 6);
    addControlObject("Filter Spacing Fixed Values " + QString::number(i1 + 1), MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 7);
    addControlObject("Filter Key Tracking Fixed Values " + QString::number(i1 + 1), MAX_SCENES * MAX_SOURCES + 4, 9 * i1 + 8);
  }

  QGroupBox *masterBox = new QGroupBox("Master");
  QVBoxLayout *masterLayout = new QVBoxLayout;
  masterBox->setLayout(masterLayout);
  masterVolume = new ModSlider(model, Qt::Horizontal);
  masterLayout->addWidget(masterVolume); 
  masterVolume->setMinimum(0);
  masterVolume->setMaximum(1000);
  masterVolume->setTickInterval(1);
  QObject::connect(masterVolume, SIGNAL(valueChanged(int)), this, SLOT(setMasterVolume(int)));
  commonMainLayout->addWidget(masterBox);
  addControlObject("Master Volume", 0, 0);
  masterVolume->setValue(300);
  
  QHBoxLayout *pitchLFOlayout = new QHBoxLayout;
  QVBoxLayout *pitchLFOlayoutL = new QVBoxLayout;
  QVBoxLayout *pitchLFOlayoutR = new QVBoxLayout;
  QGroupBox *pitchLFObox = new QGroupBox("Pitch");
  QWidget *pitchLFOboxL = new QWidget;
  QWidget *pitchLFOboxR = new QWidget;
  pitchLFOboxL->setLayout(pitchLFOlayoutL);
  pitchLFOboxR->setLayout(pitchLFOlayoutR);
  pitchLFOlayout->setContentsMargins(0, 10, 0, 10);
  pitchLFOlayout->addWidget(pitchLFOboxL);
  pitchLFOlayout->addWidget(pitchLFOboxR);
  QLabel *pitchLFOfreqLabel = new QLabel("LFO Freq ");
  QLabel *pitchLFOdepthLabel = new QLabel("LFO Depth ");
  pitchLFOfreq = new ModSlider(model, Qt::Horizontal); 
  pitchLFOdepth = new ModSlider(model, Qt::Horizontal);
  pitchLFOfreq->setMinimum(0);
  pitchLFOfreq->setMaximum(1000);
  pitchLFOfreq->setTickInterval(1);
  pitchLFOdepth->setMinimum(0);
  pitchLFOdepth->setMaximum(1000);
  pitchLFOdepth->setTickInterval(1);
  pitchLFOlayoutL->addWidget(pitchLFOfreqLabel);
  pitchLFOlayoutL->addWidget(pitchLFOfreq);
  pitchLFOlayoutR->addWidget(pitchLFOdepthLabel);
  pitchLFOlayoutR->addWidget(pitchLFOdepth);
  pitchLFObox->setLayout(pitchLFOlayout); 
  commonMainLayout->addWidget(pitchLFObox);
  addControlObject("LFO Freq", 0, 1);
  addControlObject("LFO Depth", 0, 2);

  evenOddBox = new QGroupBox("Spectrum 1");
  evenOddLayout = new QVBoxLayout;
  evenOddBox->setLayout(evenOddLayout);
  for (i1 = 0; i1 < 3; i1++) {
    evenOddSlider[i1] = new ModSlider(model, Qt::Horizontal);
    evenOddSlider[i1]->setMinimum(0);
    evenOddSlider[i1]->setMaximum(1000); 
    evenOddSlider[i1]->setTickInterval(1);
  }
  evenOddLayout->addWidget(new QLabel("Even/Odd"));
  evenOddLayout->addWidget(evenOddSlider[0]);
  QObject::connect(evenOddSlider[0], SIGNAL(valueChanged(int)), this, SLOT(setEvenOdd(int)));
  addControlObject("Even/Odd 1", MAX_SCENES * MAX_SOURCES + 1, 0);
  evenOddSlider[0]->setValue(500); 
  oscMainLayout->addWidget(evenOddBox);   

  for (i1 = 0; i1 < 2; i1++) {
    spectrumLayout[i1] = new QVBoxLayout;
    spectrumHLayout[i1] = new QHBoxLayout;
    spectrumLayoutL[i1] = new QVBoxLayout;
    spectrumLayoutR[i1] = new QVBoxLayout;
    spectrumBox[i1] = new QGroupBox("Spectrum " + QString::number(i1 + 2));
    spectrumBox[i1]->setLayout(spectrumLayout[i1]); 
    spectrumHFrame[i1] = new QWidget;
    spectrumBoxL[i1] = new QWidget;
    spectrumBoxR[i1] = new QWidget;
    spectrumHFrame[i1]->setLayout(spectrumHLayout[i1]);
    spectrumHLayout[i1]->setContentsMargins(0, 10, 0, 10);
    spectrumLayoutL[i1]->setContentsMargins(0, 10, 0, 10);
    spectrumLayoutR[i1]->setContentsMargins(0, 10, 0, 10);
    spectrumBoxL[i1]->setLayout(spectrumLayoutL[i1]);
    spectrumBoxR[i1]->setLayout(spectrumLayoutR[i1]);
    spectrumLayout[i1]->addWidget(spectrumHFrame[i1]);
    spectrumHLayout[i1]->addWidget(spectrumBoxL[i1]);
    spectrumHLayout[i1]->addWidget(new QWidget); // Dummy
    spectrumHLayout[i1]->addWidget(new QWidget); // Dummy
    spectrumHLayout[i1]->addWidget(spectrumBoxR[i1]);
    spectrumHLayout[i1]->addWidget(new QWidget); // Dummy
    spectrumHLayout[i1]->addWidget(new QWidget); // Dummy
    spectrumComboLayout[i1] = new QVBoxLayout; 
    spectrumComboFrame[i1] = new QWidget;
    spectrumComboFrame[i1]->setLayout(spectrumComboLayout[i1]);
    spectrumHLayout[i1]->addWidget(spectrumComboFrame[i1]);
    spectrumOffsetCombo[i1] = new QComboBox;
    spectrumSpacingCombo[i1] = new QComboBox;
    spectrumComboLayout[i1]->addWidget(new QLabel("Offset"));
    spectrumComboLayout[i1]->addWidget(spectrumOffsetCombo[i1]);
    spectrumComboLayout[i1]->addWidget(new QLabel("Spacing"));
    spectrumComboLayout[i1]->addWidget(spectrumSpacingCombo[i1]);
    spectrumOffsetCombo[i1]->addItem("Free");
    spectrumSpacingCombo[i1]->addItem("Free");
    for (i2 = 0; i2 < 9; i2++) {
      spectrumOffsetCombo[i1]->addItem(QString::number(i2));
    }
    for (i2 = 1; i2 < 9; i2++) {
      spectrumSpacingCombo[i1]->addItem(QString::number((double)i2 / 4.0, 'f', 2));
    }
    spectrumLabel[4 * i1] = new QLabel("Offset Coarse");
    spectrumLabel[4 * i1 + 1] = new QLabel("Spacing Coarse");
    spectrumLabel[4 * i1 + 2] = new QLabel("Offset Fine");
    spectrumLabel[4 * i1 + 3] = new QLabel("Spacing Fine");
    spectrumOffset[2 * i1] = new ModSlider(model, Qt::Horizontal); 
    spectrumSpacing[2 * i1] = new ModSlider(model, Qt::Horizontal);
    spectrumOffset[2 * i1 + 1] = new ModSlider(model, Qt::Horizontal); 
    spectrumSpacing[2 * i1 + 1] = new ModSlider(model, Qt::Horizontal);
    spectrumOffset[2 * i1]->setMinimum(0);
    spectrumOffset[2 * i1]->setMaximum(1000);
    spectrumOffset[2 * i1]->setTickInterval(1);
    spectrumSpacing[2 * i1]->setMinimum(0);
    spectrumSpacing[2 * i1]->setMaximum(1000);
    spectrumSpacing[2 * i1]->setTickInterval(1);
    spectrumOffset[2 * i1 + 1]->setMinimum(-1000);
    spectrumOffset[2 * i1 + 1]->setMaximum(1000);
    spectrumOffset[2 * i1 + 1]->setTickInterval(1);
    spectrumSpacing[2 * i1 + 1]->setMinimum(-1000);
    spectrumSpacing[2 * i1 + 1]->setMaximum(1000);
    spectrumSpacing[2 * i1 + 1]->setTickInterval(1);
    spectrumLayoutL[i1]->addWidget(spectrumLabel[4 * i1]);
    spectrumLayoutL[i1]->addWidget(spectrumOffset[2 * i1]);
    spectrumLayoutL[i1]->addWidget(spectrumLabel[4 * i1 + 2]);
    spectrumLayoutL[i1]->addWidget(spectrumOffset[2 * i1 + 1]);
    spectrumLayoutR[i1]->addWidget(spectrumLabel[4 * i1 + 1]);
    spectrumLayoutR[i1]->addWidget(spectrumSpacing[2 * i1]);
    spectrumLayoutR[i1]->addWidget(spectrumLabel[4 * i1 + 3]);
    spectrumLayoutR[i1]->addWidget(spectrumSpacing[2 * i1 + 1]);
    spectrumLayout[i1]->addWidget(new QLabel("Even/Odd"));
    spectrumLayout[i1]->addWidget(evenOddSlider[i1 + 1]);
    oscMainLayout->addWidget(spectrumBox[i1]);
    QObject::connect(spectrumOffset[2 * i1], SIGNAL(valueChanged(int)), this, SLOT(setSpectrumOffset(int)));
    QObject::connect(spectrumOffset[2 * i1 + 1], SIGNAL(valueChanged(int)), this, SLOT(setSpectrumOffset(int)));
    QObject::connect(spectrumSpacing[2 * i1], SIGNAL(valueChanged(int)), this, SLOT(setSpectrumSpacing(int)));
    QObject::connect(spectrumSpacing[2 * i1 + 1], SIGNAL(valueChanged(int)), this, SLOT(setSpectrumSpacing(int)));
    QObject::connect(evenOddSlider[1 + i1], SIGNAL(valueChanged(int)), this, SLOT(setEvenOdd(int)));
    QObject::connect(spectrumOffsetCombo[i1], SIGNAL(currentIndexChanged(int)), this, SLOT(setSpectrumOffsetCombo(int)));
    QObject::connect(spectrumSpacingCombo[i1], SIGNAL(currentIndexChanged(int)), this, SLOT(setSpectrumSpacingCombo(int)));
    addControlObject(spectrumLabel[4 * i1]->text() + " " + QString::number(i1 + 2), MAX_SCENES * MAX_SOURCES + 1, 7 * i1 + 1);
    addControlObject(spectrumLabel[4 * i1 + 2]->text() + " " + QString::number(i1 + 2), MAX_SCENES * MAX_SOURCES + 1, 7 * i1 + 2);
    addControlObject(spectrumLabel[4 * i1 + 1]->text() + " " + QString::number(i1 + 2), MAX_SCENES * MAX_SOURCES + 1, 7 * i1 + 3);
    addControlObject(spectrumLabel[4 * i1 + 3]->text() + " " + QString::number(i1 + 2), MAX_SCENES * MAX_SOURCES + 1, 7 * i1 + 4);
    addControlObject("Offset Fixed Values " + QString::number(i1 + 2), MAX_SCENES * MAX_SOURCES + 1, 7 * i1 + 5);
    addControlObject("Spacing Fixed Values "  + QString::number(i1 + 2), MAX_SCENES * MAX_SOURCES + 1, 7 * i1 + 6);
    addControlObject("Even/Odd " + QString::number(i1 + 2), MAX_SCENES * MAX_SOURCES + 1, 7 * i1 + 7);
    evenOddSlider[i1 + 1]->setValue(500); 
  }
  oscMainLayout->setSpacing(40); 
  oscMainLayout->setContentsMargins(10, 40, 10, 40); 
  for (i1 = 0; i1 < 2; i1++) {
    addControlFolder("Filter " + QString::number(i1 + 1) + " Modulation");
    for (i2 = 0; i2 < 20; i2++) {
      addControlObject(filterModLongLabels[i1][i2], MAX_SCENES * MAX_SOURCES + 5 + i1, i2);
    }
  }

  QVBoxLayout *morphingLayout = new QVBoxLayout;
  QGroupBox *morphingBox = new QGroupBox("Scene Morphing");
  morphingBox->setLayout(morphingLayout);
  QLabel *morphingLabel = new QLabel("Scene 1 <---> Scene 2");
  morphingSlider = new ModSlider(model, Qt::Horizontal); 
  morphingSlider->setMinimum(0);
  morphingSlider->setMaximum(1000);
  morphingSlider->setTickInterval(1);
  morphingLayout->addWidget(morphingLabel);
  morphingLayout->addWidget(morphingSlider);
  commonMainLayout->addWidget(morphingBox);
  addControlObject("Scene 1 <---> Scene 2", 0, 3);

  compressorLabels[0] = "Linear Level";
  compressorLabels[1] = "Transition Width";
  compressorLabels[2] = "Compression";
  compressorLabels[3] = "Antialiasing";
  compressorLayout = new QVBoxLayout;
  compressorBox = new QGroupBox("Compressor");
  compressorBox->setLayout(compressorLayout);
  for (i1 = 0; i1 < 4; i1++) {
    QLabel *compressorLabel = new QLabel(compressorLabels[i1]);
    compressorSlider[i1] = new ModSlider(model, Qt::Horizontal); 
    compressorSlider[i1]->setMinimum(0);
    compressorSlider[i1]->setMaximum(1000);
    compressorSlider[i1]->setTickInterval(1);
    compressorLayout->addWidget(compressorLabel);
    compressorLayout->addWidget(compressorSlider[i1]);
    addControlObject("Compressor "+compressorLabels[i1], 0, 4+i1);
    QObject::connect(compressorSlider[i1], SIGNAL(valueChanged(int)), this, SLOT(setCompressor(int)));
  }  
  for (i1 = 0; i1 < 4; i1++) {
    compressorSlider[i1]->setValue(500);
  }  
  commonMainLayout->addWidget(compressorBox);
  panLayout = new QVBoxLayout;  
  panBox = new QGroupBox("Panning");
  panBox->setLayout(panLayout);   
  panLabels[0] = "Pan Center";
  panLabels[1] = "Pan Width";
  panLabels[2] = "Pan Mode";
  QHBoxLayout *panModeLayout = new QHBoxLayout;
  panModeLayout->setContentsMargins(0, 10, 0, 10);
  QWidget *panModeFrame = new QWidget;
  panModeFrame->setLayout(panModeLayout);
  QVBoxLayout *panModeLayoutL = new QVBoxLayout;
  QVBoxLayout *panModeLayoutR = new QVBoxLayout;
  panModeLayoutL->setContentsMargins(0, 0, 0, 0);
  panModeLayoutR->setContentsMargins(0, 0, 0, 0);
  QWidget *panModeFrameL = new QWidget;
  QWidget *panModeFrameR = new QWidget;
  panModeFrameL->setLayout(panModeLayoutL);
  panModeFrameR->setLayout(panModeLayoutR);
  for (i1 = 0; i1 < 2; i1++) {
    QLabel *panLabel = new QLabel(panLabels[i1]);
    panSlider[i1] = new ModSlider(model, Qt::Horizontal); 
    panSlider[i1]->setMinimum(0);
    panSlider[i1]->setMaximum(1000);
    panSlider[i1]->setTickInterval(1);
    if (i1) {
      panLayout->addWidget(panModeFrame);
      panModeLayout->addWidget(panModeFrameL);
      panModeLayout->addWidget(new QWidget); // Dummy
      panModeLayout->addWidget(new QWidget); // Dummy
      panModeLayout->addWidget(panModeFrameR);
      panModeLayoutL->addWidget(panLabel);
      panModeLayoutL->addWidget(panSlider[i1]);
    } else {
      panLayout->addWidget(panLabel);
      panLayout->addWidget(panSlider[i1]);
    }  
    //addControlObject("Compressor "+panLabels[i1], 0, 4+i1);
    addControlObject(panLabels[i1], 0, 8+i1);
    QObject::connect(panSlider[i1], SIGNAL(valueChanged(int)), this, SLOT(setPan(int)));
  }  
  panSlider[0]->setValue(500);
  panMode = new QComboBox;
  panMode->addItem("Pitch Up");
  panMode->addItem("Pitch Down");
  panMode->addItem("Alternating Up");
  panMode->addItem("Alternating Down");
  panModeLayoutR->addWidget(new QLabel(panLabels[2]));
  panModeLayoutR->addWidget(panMode);
  addControlObject(panLabels[2], 0, 8+2);
  QObject::connect(panMode, SIGNAL(currentIndexChanged(int)), this, SLOT(setPanMode(int)));
  commonMainLayout->addWidget(panBox);
  //commonMainLayout->addWidget(new QWidget); // Dummy
  commonMainLayout->setContentsMargins(10, 20, 10, 20);
  QObject::connect(pitchLFOfreq, SIGNAL(valueChanged(int)), this, SLOT(setPitchLFOfreq(int)));
  QObject::connect(pitchLFOdepth, SIGNAL(valueChanged(int)), this, SLOT(setPitchLFOdepth(int)));
  QObject::connect(morphingSlider, SIGNAL(valueChanged(int)), this, SLOT(setMorphing(int)));
  QObject::connect(jackProcessor, SIGNAL(MIDI_controlEvent(int, unsigned char, unsigned int, int, int)), this, SLOT(processMidiController(int, unsigned char, unsigned int, int, int)));
  morphingSlider->setValue(500);
  modifiedFlag = false;
  QObject::connect(mainTab, SIGNAL(currentChanged(int)), this, SLOT(mainTabChanged(int)));
  voiceRangeSet();
  jackProcessor->initJack();
  synth->start();
}

Gui::~Gui() {

}

void Gui::loadAll() {

  int i1;
  QString fn, qs;
  QFile f;
  int vStart, vEnd, currentIndex;
  VoiceData *vd;
    
  model->getEditRange(vStart, vEnd);
  QFileDialog *dialog = new QFileDialog;
  fn = dialog->getOpenFileName();
  if (!fn.isEmpty()) {
    f.setFileName(fn);
    if (f.exists()) {
      voiceFileName = fn;
      voiceFileLabel->setText(fn);
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFileName(fn);
      }  
      currentIndex = voiceTree->indexOfTopLevelItem(voiceTree->currentItem());
      if (currentIndex < 0) {
        currentIndex = vStart;
      }
      voiceTree->clear();
      for (i1 = 0; i1 < model->getPoly(); i1++) {
        vd = model->getVoiceData(i1);
        QTreeWidgetItem *item = new QTreeWidgetItem(voiceTree);
        qs = QString::number(i1 + 1);
        item->setText(0, qs);
        qs = QString::number(vd->getChannel() + 1);
        item->setText(1, qs);
        item->setText(2, vd->getFileName());
        if (i1 == currentIndex) {
          voiceTree->setCurrentItem(item);
        }
      }  
      loadVoice(vStart);
    }
  }    
}                

void Gui::saveAll() {

  int i1;
  QString fn, qs;
  FILE *f;
  int vStart, vEnd, currentIndex;
  VoiceData *vd;
      
  model->getEditRange(vStart, vEnd);
  QFileDialog *dialog = new QFileDialog;
  fn = dialog->getSaveFileName();
  if (!fn.isEmpty()) {
    if (!(f = fopen(fn.toLatin1(), "w"))) {
      QMessageBox::warning(this, "Add64", "Could not save file.");
    } else {
      voiceFileName = fn;
      voiceFileLabel->setText(fn);
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFileName(fn);  
      }
      currentIndex = voiceTree->indexOfTopLevelItem(voiceTree->currentItem());
      if (currentIndex < 0) {
        currentIndex = vStart;
      }
      voiceTree->clear();
      for (i1 = 0; i1 < model->getPoly(); i1++) {
        vd = model->getVoiceData(i1);
        QTreeWidgetItem *item = new QTreeWidgetItem(voiceTree);
        qs = QString::number(i1 + 1);
        item->setText(0, qs);
        qs = QString::number(vd->getChannel() + 1);
        item->setText(1, qs);
        item->setText(2, vd->getFileName());
        if (i1 == currentIndex) {
          voiceTree->setCurrentItem(item);
        }
      }
      saveVoice(vStart);  
    }  
  }   
}

void Gui::loadMidi() {

  int d1, d2, d3, d4, d5;
  QString fn, qs;
  char buf[256];
  QFile f;
  QTreeWidgetItem *m;

  QFileDialog *dialog = new QFileDialog;
  fn = dialog->getOpenFileName();
  if (!fn.isEmpty()) {
    f.setFileName(fn);
    if (f.exists()) {
      f.open(QIODevice::ReadOnly);
      midiControllerTree->clear();
      model->clearMidiControllerList();
      m = NULL;
      while (!f.atEnd()) {
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        if (qs.contains("_MIDI_")) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          d1 = qs.toInt();
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          d2 = qs.toInt();
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          d3 = qs.toInt();
          addMidiController(d1, d2, d3);
          m = midiControllerTree->topLevelItem(midiControllerTree->topLevelItemCount() - 1);
        } else {
          d1 = qs.toInt();
          f.readLine(buf, sizeof(buf)); 
          qs = QString(buf);
          d2 = qs.toInt();
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          d3 = qs.toInt();
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          d4 = qs.toInt();
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          d5 = qs.toInt();
          connectController(m, d1, d2, d3, d4, d5);
          if (m != NULL) {
            midiControllerTree->expandItem(m);
          }  
        }
      }
    }
  }  
}

void Gui::saveMidi() {

  int i1, i2;
  QString fn, qs;
  FILE *f;
  midiController m;
  controlObject co;

  QFileDialog *dialog = new QFileDialog;
  fn = dialog->getSaveFileName();
  if (!fn.isEmpty()) {
    if (!(f = fopen(fn.toLatin1(), "w"))) {
      QMessageBox::warning(this, "Add64", "Could not save file.");
    } else {
      for (i1 = 0; i1 < model->midiControllerCount(); i1++) {
        m = model->getMidiController(i1);
        fprintf(f, "_MIDI_\n");
        fprintf(f, "%d\n", m.type);
        fprintf(f, "%d\n", m.eventChannel);
        fprintf(f, "%d\n", m.eventParam);
        for (i2 = 0; i2 < model->connectedObjectCount(i1); i2++) {
          co = model->getConnectedObject(i1, i2);
          fprintf(f, "%d\n", co.objectType);
          fprintf(f, "%d\n", co.objectIndex);
          fprintf(f, "%d\n", co.modDepth);
          fprintf(f, "%d\n", co.full);
          fprintf(f, "%d\n", co.inverse);
        }
      }
      fclose(f);
    }  
  }   
}

void Gui::reScale(int value) {

  int i1;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);    
  for (i1 = 0; i1 < MAX_PARAMS; i1++) {
    if (harmonicScale[i1] == sender()) {
      for (int vl = vStart; vl <= vEnd; vl++) {
        val = ((ModSlider *)sender())->getValue(vl);
        model->getVoiceData(vl)->setGroupScale(model->getEditGroup(), i1, val); // TODO Check if model->getEditGroup() correct
      }
      break;
    }
  }  
}

void Gui::selectGroup(int p_group) {

  int i;

  model->setEditGroup(p_group);
  for (i = 0; i < MAX_PARAMS; i++) {
    harmonicScale[i]->setLayer(p_group); 
  }  
  harmonics[harmonicTab->currentIndex()]->update(); // TODO Check if this should stay in model or move to VoiceData
}

void Gui::updateTab(int id) {

  harmonicScale[id]->setLayer(model->getEditGroup());
  harmonicScale[id]->updateSlider();
}

void Gui::setGlobalScale(int objectType, int objectIndex, int voiceIndex, double modValue, int value) {

  double val;
  int d1, d2;
  bool modFlag;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  modFlag = modValue < 1234;
  if ((harmonicScale[objectIndex]->getLayer() == (objectType - 1))
       && (objectIndex == harmonicTab->currentIndex())) {
    if (modFlag) {    
      harmonicScale[objectIndex]->setValues(voiceIndex, modValue, 1234);
    } else {
      harmonicScale[objectIndex]->setValues(voiceIndex, 1234, (int)((double)value * 2000.0 / 127.0 - 1000.0));
    }  
  } else {
    if (modFlag) { 
      harmonicScale[objectIndex]->writeBufValue(objectType - 1, 1234, voiceIndex, modValue);
    } else {
      harmonicScale[objectIndex]->writeBufValue(objectType - 1, (int)((double)value * 2000.0 / 127.0 - 1000.0), voiceIndex, 1234);
    }
    for (int vl = vStart; vl <= vEnd; vl++) {
      harmonicScale[objectIndex]->readBufValue(objectType - 1, d1, vl, d2);
      val = d1 + d2;
//    printf("setGlobalScale %d  %f\n", value, modValue);
      if (val < -1000) {
        val = -1000;
      } else if (val > 1000) {
        val = 1000;
      }
      model->getVoiceData(vl)->setGroupScale(objectType - 1, objectIndex, (int)val); 
    }  
  }  
}

void Gui::setCommon(int objectType, int objectIndex, int voiceIndex, double modValue, int value) {

  double c;

  switch (objectIndex) {
    case 0:
      c = (masterVolume->maximum() - masterVolume->minimum()) / 127.0;
      masterVolume->setValues(voiceIndex, modValue, masterVolume->minimum() + (int)(c * (double)value));
      break;
    case 1:
      c = (pitchLFOfreq->maximum() - pitchLFOfreq->minimum()) / 127.0;
      pitchLFOfreq->setValues(voiceIndex, modValue, pitchLFOfreq->minimum() + (int)(c * (double)value));
      break;
    case 2:
      c = (pitchLFOdepth->maximum() - pitchLFOdepth->minimum()) / 127.0;
      pitchLFOdepth->setValues(voiceIndex, modValue, pitchLFOdepth->minimum() + (int)(c * (double)value));
      break;
    case 3:
      c = (morphingSlider->maximum() - morphingSlider->minimum()) / 127.0;
      morphingSlider->setValues(voiceIndex, modValue, morphingSlider->minimum() + (int)(c * (double)value));
      break;
    case 4:
      c = (compressorSlider[0]->maximum() - compressorSlider[0]->minimum()) / 127.0;
      compressorSlider[0]->setValues(voiceIndex, modValue, compressorSlider[0]->minimum() + (int)(c * (double)value));
      break;
    case 5:
      c = (compressorSlider[1]->maximum() - compressorSlider[1]->minimum()) / 127.0;
      compressorSlider[1]->setValues(voiceIndex, modValue, compressorSlider[1]->minimum() + (int)(c * (double)value));
      break;
    case 6:
      c = (compressorSlider[2]->maximum() - compressorSlider[2]->minimum()) / 127.0;
      compressorSlider[2]->setValues(voiceIndex, modValue, compressorSlider[2]->minimum() + (int)(c * (double)value));
      break;
    case 7:
      c = (compressorSlider[3]->maximum() - compressorSlider[3]->minimum()) / 127.0;
      compressorSlider[3]->setValues(voiceIndex, modValue, compressorSlider[3]->minimum() + (int)(c * (double)value));
      break;
    case 8:
      c = (panSlider[0]->maximum() - panSlider[0]->minimum()) / 127.0;
      panSlider[0]->setValues(voiceIndex, modValue, panSlider[0]->minimum() + (int)(c * (double)value));
      break;
    case 9:
      c = (panSlider[1]->maximum() - panSlider[1]->minimum()) / 127.0;
      panSlider[1]->setValues(voiceIndex, modValue, panSlider[1]->minimum() + (int)(c * (double)value));
      break;
    case 10:
      c = ((panMode->count() - 1.0) / 127.0);
      panMode->setCurrentIndex((int)(c * (double)value));
      break;
  }
}

void Gui::setOscillators(int objectType, int objectIndex, int voiceIndex, double modValue, int value) {

  double c;

  switch (objectIndex) {
    case 0:
      c = (evenOddSlider[0]->maximum() - evenOddSlider[0]->minimum()) / 127.0;
      evenOddSlider[0]->setValues(voiceIndex, modValue, evenOddSlider[0]->minimum() + (int)(c * (double)value));
      break;
    case 1:
      c = (spectrumOffset[0]->maximum() - spectrumOffset[0]->minimum()) / 127.0;
      spectrumOffset[0]->setValues(voiceIndex, modValue, spectrumOffset[0]->minimum() + (int)(c * (double)value));
      break;
    case 2:
      c = (spectrumOffset[1]->maximum() - spectrumOffset[1]->minimum()) / 127.0;
      spectrumOffset[1]->setValues(voiceIndex, modValue, spectrumOffset[1]->minimum() + (int)(c * (double)value));
      break;
    case 3:
      c = (spectrumSpacing[0]->maximum() - spectrumSpacing[0]->minimum()) / 127.0;
      spectrumSpacing[0]->setValues(voiceIndex, modValue, spectrumSpacing[0]->minimum() + (int)(c * (double)value));
      break;
    case 4:
      c = (spectrumSpacing[1]->maximum() - spectrumSpacing[1]->minimum()) / 127.0;
      spectrumSpacing[1]->setValues(voiceIndex, modValue, spectrumSpacing[1]->minimum() + (int)(c * (double)value));
      break;
    case 5:
      c = ((spectrumOffsetCombo[0]->count() - 1.0) / 127.0);
      spectrumOffsetCombo[0]->setCurrentIndex((int)(c * (double)value));
      break;
    case 6:
      c = ((spectrumSpacingCombo[0]->count() - 1.0) / 127.0);
      spectrumSpacingCombo[0]->setCurrentIndex((int)(c * (double)value));
      break;
    case 7: 
      c = (evenOddSlider[1]->maximum() - evenOddSlider[1]->minimum()) / 127.0;
      evenOddSlider[1]->setValues(voiceIndex, modValue, evenOddSlider[1]->minimum() + (int)(c * (double)value));
      break;
    case 8: 
      c = (spectrumOffset[2]->maximum() - spectrumOffset[2]->minimum()) / 127.0;
      spectrumOffset[2]->setValues(voiceIndex, modValue, spectrumOffset[2]->minimum() + (int)(c * (double)value));
      break;
    case 9:
      c = (spectrumOffset[3]->maximum() - spectrumOffset[3]->minimum()) / 127.0;
      spectrumOffset[3]->setValues(voiceIndex, modValue, spectrumOffset[3]->minimum() + (int)(c * (double)value));
      break;
    case 10:
      c = (spectrumSpacing[2]->maximum() - spectrumSpacing[2]->minimum()) / 127.0;
      spectrumSpacing[2]->setValues(voiceIndex, modValue, spectrumSpacing[2]->minimum() + (int)(c * (double)value));
      break;
    case 11:
      c = (spectrumSpacing[3]->maximum() - spectrumSpacing[3]->minimum()) / 127.0;
      spectrumSpacing[3]->setValues(voiceIndex, modValue, spectrumSpacing[3]->minimum() + (int)(c * (double)value));
      break;
    case 12:
      c = ((spectrumOffsetCombo[1]->count() - 1.0) / 127.0);
      spectrumOffsetCombo[1]->setCurrentIndex((int)(c * (double)value));
      break;
    case 13:
      c = ((spectrumSpacingCombo[1]->count() - 1.0) / 127.0);
      spectrumSpacingCombo[1]->setCurrentIndex((int)(c * (double)value));
      break;
    case 14:
      c = (evenOddSlider[2]->maximum() - evenOddSlider[2]->minimum()) / 127.0;
      evenOddSlider[2]->setValues(voiceIndex, modValue, evenOddSlider[2]->minimum() + (int)(c * (double)value));
      break;
  }
}

void Gui::setPitchLFOfreq(int value) {

  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (int vl = vStart; vl <= vEnd; vl++) {
    val = ((ModSlider *)sender())->getValue(vl);
    model->getVoiceData(vl)->setLfoFreq(0.1 + val / 50.0);
  }  
}

void Gui::setPitchLFOdepth(int value) {

  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (int vl = vStart; vl <= vEnd; vl++) {
    val = ((ModSlider *)sender())->getValue(vl);
    model->getVoiceData(vl)->setLfoDepth(val / 1500.0);
  }  
}

void Gui::setEvenOdd(int value) {

  int i;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 3; i++) {
    if (evenOddSlider[i] == sender()) {
      for (int vl = vStart; vl <= vEnd; vl++) { 
        val = ((ModSlider *)sender())->getValue(vl);
        model->getVoiceData(vl)->setEvenOdd(i, val / 1000.0);
      }
      break;      
    }
  }            
}

void Gui::setSpectrumOffset(int value) {

  int i;
  int vStart, vEnd;
  
//  printf("%d\n", value);
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 4; i++) {
    if (spectrumOffset[i] == sender()) {
      switch (i) {
        case 0:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setDelta(0, 0.25 + spectrumOffset[0]->getValue(vl) / 125.0 + (spectrumOffset[1]->getValue(vl)) / 2000.0);
          }  
          break;
        case 1:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setDelta(0, 0.25 + spectrumOffset[0]->getValue(vl) / 125.0 + (spectrumOffset[1]->getValue(vl)) / 2000.0);
          }
          break;
        case 2:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setDelta(2, 0.25 + spectrumOffset[2]->getValue(vl) / 125.0 + (spectrumOffset[3]->getValue(vl)) / 2000.0);
          }  
          break;
        case 3:
          for (int vl = vStart; vl <= vEnd; vl++) { 
            model->getVoiceData(vl)->setDelta(2, 0.25 + spectrumOffset[2]->getValue(vl) / 125.0 + (spectrumOffset[3]->getValue(vl)) / 2000.0);
          }
          break;
      }
      break;
    }
  }
}

void Gui::setSpectrumSpacing(int value) {

  int i;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 4; i++) {
    if (spectrumSpacing[i] == sender()) {
      switch (i) {
        case 0:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setDelta(1, 0.25 + spectrumSpacing[0]->getValue(vl) / 500.0 + 0.249e-3 * spectrumSpacing[1]->getValue(vl));
          }
          break;
        case 1:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setDelta(1, 0.25 + spectrumSpacing[0]->getValue(vl) / 500.0 + 0.249e-3 * spectrumSpacing[1]->getValue(vl));
          }
          break;
        case 2:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setDelta(3, 0.25 + spectrumSpacing[2]->getValue(vl) / 500.0 + 0.249e-3 * spectrumSpacing[3]->getValue(vl));
          }
          break;
        case 3:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setDelta(3, 0.25 + spectrumSpacing[2]->getValue(vl) / 500.0 + 0.249e-3 * spectrumSpacing[3]->getValue(vl));
          }
          break;
      }
      break;
    }
  }
}

void Gui::setSpectrumOffsetCombo(int index) {

  int i;

  if (index) {  
    for (i = 0; i < 2; i++) {
      if (spectrumOffsetCombo[i] == sender()) {
        switch (i) {
          case 0:
            if (index > 1) {
              spectrumOffset[2 * i]->setValues(0, (int)round(125.0 * ((double)index - 1.0 - 0.25)));
            } else {
              spectrumOffset[2 * i]->setValues(0, (int)(125.0 * 0.25));
            } 
            spectrumOffset[2 * i + 1]->setValues(0, 0);
            break;
          case 1:
            if (index > 1) {
              spectrumOffset[2 * i]->setValues(0, (int)round(125.0 * ((double)index - 1.0 - 0.25)));
            } else {
              spectrumOffset[2 * i]->setValues(0, (int)(125.0 * 0.25));
            } 
            spectrumOffset[2 * i + 1]->setValues(0, 0);
            break;
        }
        break;
      }
    }
  }  
}

void Gui::setSpectrumSpacingCombo(int index) {

  int i;

  if (index) {  
    for (i = 0; i < 2; i++) {
      if (spectrumSpacingCombo[i] == sender()) {
        switch (i) {
          case 0:
            if (index > 1) {
              spectrumSpacing[2 * i]->setValues(0, (int)round(125.0 * ((double)index - 1.0)));
            } else {
              spectrumSpacing[2 * i]->setValues(0, 0);
            } 
            spectrumSpacing[2 * i + 1]->setValues(0, 0);
            break;
          case 1:
            if (index > 1) {
              spectrumSpacing[2 * i]->setValues(0, (int)round(125.0 * ((double)index - 1.0)));
            } else {
              spectrumSpacing[2 * i]->setValues(0, 0);
            } 
            spectrumSpacing[2 * i + 1]->setValues(0, 0);
            break;
        }
        break;
      }
    }
  }  
}

void Gui::setMorphing(int value) {

  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (int vl = vStart; vl <= vEnd; vl++) {
    val = ((ModSlider *)sender())->getValue(vl);
    model->getVoiceData(vl)->setMorphing(val / 1000.0);
  }
}

void Gui::closeGui() {

}

void Gui::processMidiController(int type, unsigned char eventChannel,
                                unsigned int eventParam, int value, int voiceIndex) {
  if (!hasMidiController(type, eventChannel, eventParam, value, voiceIndex)) {
    addMidiController(type, eventChannel, eventParam);
  }
}                                

bool Gui::hasMidiController(int type, unsigned char eventChannel,
                            unsigned int eventParam, int value, int voiceIndex) {

  int i1, i2;
  midiController m;
  double modValue, val;
  
  i1 = model->hasMidiController(type, eventChannel, eventParam); 
  if (i1 >= 0) {
    m = model->getMidiController(i1);
    if (followMidi) {
      midiControllerTree->setCurrentItem(m.treeWidgetItem);
    }
    for (i2 = 0; i2 < m.connectedObjects.count(); i2++) {
      controlObject co = model->getConnectedObject(i1, i2);
      if (co.full) {
        val = value;
        modValue = 1234; // do not update modValue in ModSlider
      } else {
        modValue = (co.inverse) ? -(double)co.modDepth * (double)value/127.0 : (double)co.modDepth * (double)value/127.0;
        // printf("modDepth: %d  value: %d  modValue: %f\n", co.modDepth, value, modValue);
        val = 1234; // do not update value in ModSlider
      }
      switch (co.objectType) {
        case 0:
          setCommon(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 1:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 2:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 3:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 4:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 5:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 6:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 7:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 8:
          setGlobalScale(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 9:
          setOscillators(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 10:
          setNoise(co.objectType, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 11:
          setMixer(co.objectIndex, voiceIndex, modValue, val);
          break;
        case 12:
          setFilter(co.objectIndex, voiceIndex, modValue, val);
          break;
        case 13:
          setModulation(0, co.objectIndex, voiceIndex, modValue, val);
          break;
        case 14:
          setModulation(1, co.objectIndex, voiceIndex, modValue, val);
          break;
      }
    }
    return(true);
  }
  return(false);    
} 

void Gui::addMidiController(int type, unsigned char eventChannel,    
                            unsigned int eventParam) {
  QString qs;                     

  switch (type) {
    case 1:
      qs = "Ch " + QString::number(eventChannel) + " - Note";
      break;
    case 2:
      qs = "Ch " + QString::number(eventChannel) + " - Velocity";
      break;
    case 3:
      qs = "Ch " + QString::number(eventChannel) + " - Controller " + QString::number(eventParam);
      break;
    case 4:
      qs = "Ch " + QString::number(eventChannel) + " - Channel Pressure";
      break;
    case 5:
      qs = "Ch " + QString::number(eventChannel) + " - Key Pressure " + QString::number(eventParam);
      break;
  }
  QTreeWidgetItem *item = new QTreeWidgetItem(midiControllerTree);
  item->setText(0, qs);
  model->addMidiController(type, eventChannel, eventParam, item);
}

void Gui::addControlFolder(QString name) {

  QTreeWidgetItem *item = new QTreeWidgetItem(controlObjectTree);
  item->setText(0, name);
}

void Gui::addControlObject(QString name, int objectType,
                           int objectIndex) {
  QString qs;                     

  QTreeWidgetItem *item = new QTreeWidgetItem;
  controlObjectTree->topLevelItem(objectType)->addChild(item);
  QTextStream(&qs) << name;
  item->setText(0, qs);
  model->addControlObject(controlObjectTree->topLevelItem(objectType)->text(0) + " - " + qs, objectType, objectIndex, item);
}

void Gui::connectController() {

  bool notInList;
  QString name;
  
  if (!midiControllerTree->topLevelItemCount() || (midiControllerTree->currentItem() == NULL) 
      || (controlObjectTree->currentItem() == NULL) || controlObjectTree->currentItem()->childCount()
      || !model->isMidiControllerItem(midiControllerTree->currentItem())) {
    return;
  }
  QTreeWidgetItem *m = midiControllerTree->currentItem();
  QTreeWidgetItem *c = controlObjectTree->currentItem();
  notInList = model->connectController(m, c);
  if (notInList) {
    QTreeWidgetItem *item = new QTreeWidgetItem(m);
    name = c->parent()->text(0) + " - " + c->text(0);
    item->setText(0, name);
    midiControllerTree->expandItem(m);
    model->setControllerItem(m, c, item);
  }
}  

void Gui::connectController(QTreeWidgetItem *midiControllerItem, int objectType, int objectIndex, int modDepth, bool full, bool inverse) {

  bool notInList;
  
  notInList = model->connectController(midiControllerItem, objectType, objectIndex, modDepth, full, inverse);
  if (notInList) {
    QTreeWidgetItem *item = new QTreeWidgetItem(midiControllerItem);
    item->setText(0, model->getControlObjectName(objectType, objectIndex));
    midiControllerTree->expandItem(midiControllerItem);
    model->setControllerItem(midiControllerItem, objectType, objectIndex, item);
  }
}  

void Gui::disconnectController() {

  bool success;

  if (!midiControllerTree->topLevelItemCount() || (midiControllerTree->currentItem() == NULL)) {
    return;
  }
  QTreeWidgetItem *m = midiControllerTree->currentItem();
  success = model->disconnectController(m->parent(), m);
  if (success) {
    m->parent()->removeChild(m);
  }  
}

void Gui::setFilterCutoff(int value) {

  int i;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 4; i++) {
    if (filterCutoff[i] == sender()) {
      switch (i) {
        case 0:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterCutoff(0, 160.0 * filterCutoff[0]->getValue(vl) / 1000.0 
                                     + 12.0 * (filterCutoff[1]->getValue(vl)-500.0) / 500.0 - 60.0);
          }
          break;
        case 1:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterCutoff(0, 160.0 * filterCutoff[0]->getValue(vl) / 1000.0 
                                     + 12.0 * (filterCutoff[1]->getValue(vl)-500.0) / 500.0 - 60.0);
          }
          break;
        case 2:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterCutoff(1, 160.0 * filterCutoff[2]->getValue(vl) / 1000.0 
                                     + 12.0 * (filterCutoff[3]->getValue(vl)-500.0) / 500.0 - 60.0);
          }
          break;
        case 3:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterCutoff(1, 160.0 * filterCutoff[2]->getValue(vl) / 1000.0 
                                     + 12.0 * (filterCutoff[3]->getValue(vl)-500.0) / 500.0 - 60.0);
          }
          break;
      }
      break;
    }
  }
}

void Gui::setFilterSpacing(int value) {

  int i;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 4; i++) {
    if (filterSpacing[i] == sender()) {
      switch (i) {
        case 0:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterSpacing(0, -0.5 + exp(2.52573 * filterSpacing[0]->getValue(vl) / 1000.0) 
                                     + 0.25 * (filterSpacing[1]->getValue(vl)-500.0) / 500.0);
          }
          break;
        case 1:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterSpacing(0, -0.5 + exp(2.52573 * filterSpacing[0]->getValue(vl) / 1000.0) 
                                     + 0.25 * (filterSpacing[1]->getValue(vl)-500.0) / 500.0);
          }
          break;
        case 2:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterSpacing(1, -0.5 + exp(2.52573 * filterSpacing[2]->getValue(vl) / 1000.0) 
                                     + 0.25 * (filterSpacing[3]->getValue(vl)-500.0) / 500.0);
          }
          break;
        case 3:
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilterSpacing(1, -0.5 + exp(2.52573 * filterSpacing[2]->getValue(vl) / 1000.0) 
                                     + 0.25 * (filterSpacing[3]->getValue(vl)-500.0) / 500.0);
          }
          break;
      }
      break;
    }
  }
}

void Gui::setFilterCutoffCombo(int index) {

  int i;

  if (index) {  
    for (i = 0; i < 2; i++) {
      if (filterCutoffCombo[i] == sender()) {
        switch (i) {
          case 0:
            filterCutoff[2 * i]->setValues(0, (int)round((1000.0 / 160.0) * (double)(12 * index - 12)));
            filterCutoff[2 * i + 1]->setValues(0, 500);
            break;
          case 1:
            filterCutoff[2 * i]->setValues(0, (int)round((1000.0 / 160.0) * (double)(12 * index - 12)));
            filterCutoff[2 * i + 1]->setValues(0, 500);
            break;
        }
        break;
      }
    }
  }  
}

void Gui::setFilterSpacingCombo(int index) {

  int i;

  if (index) {  
    for (i = 0; i < 2; i++) {
      if (filterSpacingCombo[i] == sender()) {
        switch (i) {
          case 0:
            filterSpacing[2 * i]->setValues(0, (int)round(1000.0 / 2.52573 * log(0.5 * (double)index + 0.5)));
            filterSpacing[2 * i + 1]->setValues(0, 500);
            break;
          case 1:
            filterSpacing[2 * i]->setValues(0, (int)round(1000.0 / 2.52573 * log(0.5 * (double)index + 0.5)));
            filterSpacing[2 * i + 1]->setValues(0, 500);
            break;
        }
        break;
      }
    }
  }  
}

void Gui::setFilterKeyTracking(int value) {

  int i;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 2; i++) {
    if (filterKeyTracking[i] == sender()) {
      switch (i) {
        case 0:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setFilterKeyTracking(0, val / 1000.0);
          }
          break;              
        case 1:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setFilterKeyTracking(1, val / 1000.0);
          }
          break;              
      }
      break;
    }  
  }   
}

void Gui::setFilterTrackingCombo(int index) {

  int i;

  if (index) {  
    for (i = 0; i < 2; i++) {
      if (filterTrackingCombo[i] == sender()) {
        switch (i) {
          case 0:
            filterKeyTracking[i]->setValues(0, 250 * (index - 1));
            break;
          case 1:
            filterKeyTracking[i]->setValues(0, 250 * (index - 1));
            break;
        }
        break;
      }
    }
  }  
}

void Gui::setFilterDepth(int value) {

  int i;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 2; i++) {
    if (filterDepth[i] == sender()) {
      switch (i) {
        case 0:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setFilterDepth(0, log(1.0 + (exp(1.0) - 1.0) * val / 1000.0));
          }
          break;              
        case 1:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setFilterDepth(1, log(1.0 + (exp(1.0) - 1.0) * val / 1000.0));
          }
          break;              
      }
      break;
    }  
  }   
}

void Gui::setMixer(int index, int voiceIndex, double modValue, int value) {

  double c;
  int i1, i2;
 
  i1 = index / 9;
  i2 = index % 9;
  switch (i2) {
    case 0:
      c = (filterMixSlider[i1][0]->maximum() - filterMixSlider[i1][0]->minimum()) / 127.0;
      filterMixSlider[i1][0]->setValues(voiceIndex, modValue, filterMixSlider[i1][0]->minimum() + (int)(c * (double)value));
      break;
    case 1:
      c = (filterMixSlider[i1][1]->maximum() - filterMixSlider[i1][1]->minimum()) / 127.0;
      filterMixSlider[i1][1]->setValues(voiceIndex, modValue, filterMixSlider[i1][1]->minimum() + (int)(c * (double)value));
      break;
    case 2:
      filterMixCheckBox[i1][0]->setChecked(value > 62);
      break;
    case 3:
      c = (filterMixSlider[i1][2]->maximum() - filterMixSlider[i1][2]->minimum()) / 127.0;
      filterMixSlider[i1][2]->setValues(voiceIndex, modValue, filterMixSlider[i1][2]->minimum() + (int)(c * (double)value));
      break;
    case 4:
      c = (filterMixSlider[i1][3]->maximum() - filterMixSlider[i1][3]->minimum()) / 127.0;
      filterMixSlider[i1][3]->setValues(voiceIndex, modValue, filterMixSlider[i1][3]->minimum() + (int)(c * (double)value));
      break;
    case 5:
      filterMixCheckBox[i1][1]->setChecked(value > 62);
      break;
    case 6:
      c = (filterMixSlider[i1][4]->maximum() - filterMixSlider[i1][4]->minimum()) / 127.0;
      filterMixSlider[i1][4]->setValues(voiceIndex, modValue, filterMixSlider[i1][4]->minimum() + (int)(c * (double)value));
      break;
    case 7:
      c = (filterMixSlider[i1][5]->maximum() - filterMixSlider[i1][5]->minimum()) / 127.0;
      filterMixSlider[i1][5]->setValues(voiceIndex, modValue, filterMixSlider[i1][5]->minimum() + (int)(c * (double)value));
      break;
    case 8:
      filterMixCheckBox[i1][2]->setChecked(value > 62);
      break;
  }
}

void Gui::setFilter(int index, int voiceIndex, double modValue, int value) {

  double c;
 
  switch (index) {
    case 0:
      c = (filterCutoff[0]->maximum() - filterCutoff[0]->minimum()) / 127.0;
      filterCutoff[0]->setValues(voiceIndex, modValue, filterCutoff[0]->minimum() + (int)(c * (double)value));
      break;
    case 1:
      c = (filterCutoff[1]->maximum() - filterCutoff[1]->minimum()) / 127.0;
      filterCutoff[1]->setValues(voiceIndex, modValue, filterCutoff[1]->minimum() + (int)(c * (double)value));
      break;
    case 2:
      c = (filterSpacing[0]->maximum() - filterSpacing[0]->minimum()) / 127.0;
      filterSpacing[0]->setValues(voiceIndex, modValue, filterSpacing[0]->minimum() + (int)(c * (double)value));
      break;
    case 3:
      c = (filterSpacing[1]->maximum() - filterSpacing[1]->minimum()) / 127.0;
      filterSpacing[1]->setValues(voiceIndex, modValue, filterSpacing[1]->minimum() + (int)(c * (double)value));
      break;
    case 4:
      c = (filterKeyTracking[0]->maximum() - filterKeyTracking[0]->minimum()) / 127.0;
      filterKeyTracking[0]->setValues(voiceIndex, modValue, filterKeyTracking[0]->minimum() + (int)(c * (double)value));
      break;
    case 5:
      c = (filterDepth[0]->maximum() - filterDepth[0]->minimum()) / 127.0;
      filterDepth[0]->setValues(voiceIndex, modValue, filterDepth[0]->minimum() + (int)(c * (double)value));
      break;
    case 6:
      c = ((filterCutoffCombo[0]->count() - 1.0) / 127.0);
      filterCutoffCombo[0]->setCurrentIndex((int)(c * (double)value));
      break;
    case 7:
      c = ((filterSpacingCombo[0]->count() - 1.0) / 127.0);
      filterSpacingCombo[0]->setCurrentIndex((int)(c * (double)value));
      break;
    case 8:
      c = ((filterTrackingCombo[0]->count() - 1.0) / 127.0);
      filterTrackingCombo[0]->setCurrentIndex((int)(c * (double)value));
      break;
    case 9:
      c = (filterCutoff[2]->maximum() - filterCutoff[2]->minimum()) / 127.0;
      filterCutoff[2]->setValues(voiceIndex, modValue, filterCutoff[2]->minimum() + (int)(c * (double)value));
      break;
    case 10:
      c = (filterCutoff[3]->maximum() - filterCutoff[3]->minimum()) / 127.0;
      filterCutoff[3]->setValues(voiceIndex, modValue, filterCutoff[3]->minimum() + (int)(c * (double)value));
      break;
    case 11:
      c = (filterSpacing[2]->maximum() - filterSpacing[2]->minimum()) / 127.0;
      filterSpacing[2]->setValues(voiceIndex, modValue, filterSpacing[2]->minimum() + (int)(c * (double)value));
      break;
    case 12:
      c = (filterSpacing[3]->maximum() - filterSpacing[3]->minimum()) / 127.0;
      filterSpacing[3]->setValues(voiceIndex, modValue, filterSpacing[3]->minimum() + (int)(c * (double)value));
      break;
    case 13:
      c = (filterKeyTracking[1]->maximum() - filterKeyTracking[1]->minimum()) / 127.0;
      filterKeyTracking[1]->setValues(voiceIndex, modValue, filterKeyTracking[1]->minimum() + (int)(c * (double)value));
      break;
    case 14:
      c = (filterDepth[1]->maximum() - filterDepth[1]->minimum()) / 127.0;
      filterDepth[1]->setValues(voiceIndex, modValue, filterDepth[1]->minimum() + (int)(c * (double)value));
      break;
    case 15:
      c = ((filterCutoffCombo[1]->count() - 1.0) / 127.0);
      filterCutoffCombo[1]->setCurrentIndex((int)(c * (double)value));
      break;
    case 16:
      c = ((filterSpacingCombo[1]->count() - 1.0) / 127.0);
      filterSpacingCombo[1]->setCurrentIndex((int)(c * (double)value));
      break;
    case 17:
      c = ((filterTrackingCombo[1]->count() - 1.0) / 127.0);
      filterTrackingCombo[1]->setCurrentIndex((int)(c * (double)value));
      break;
  }
}

void Gui::setFilterMix(int value) {

  int i1, i2;
  bool found;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  found = false;
  for (i1 = 0; i1 < MAX_SOURCES; i1++) {
    for (i2 = 0; i2 < 6; i2++) {
      if (filterMixSlider[i1][i2] == sender()) {
        found = true;
        break;
      }
    }      
    if (found) {
      break;
    }
  }
  switch (i2) {
    case 0:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixMorph(i1, filterMixSlider[i1][0]->getValue(vl) / 1000.0);
      }
      break;
    case 1:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixMode(i1, filterMixSlider[i1][1]->getValue(vl) / 1000.0);
      }
      break;
    case 2:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixEnvAmount(i1, filterMixSlider[i1][2]->getValue(vl) / 1000.0);
      }
      break;
    case 3:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixDelay(i1, 0.03 * (double)model->getRate() * pow(10, 2.5 * (filterMixSlider[i1][3]->getValue(vl) - 400.0) / 600.0));
      }
      break;
    case 4:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixAttack(i1, 0.1 * (double)model->getRate() * pow(10, 2.5 * (filterMixSlider[i1][4]->getValue(vl) - 400.0) / 600.0) / (double)STEP);
      }
      break;
    case 5:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixDecay(i1, 0.1 * (double)model->getRate() * pow(10, 2.5 * (filterMixSlider[i1][5]->getValue(vl) - 400.0) / 600.0) / (double)STEP);
      }
      break;
  }  
}

void Gui::filterMixToggled(int value) {

  int i1, i2;
  bool found;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  found = false;
  for (i1 = 0; i1 < MAX_SOURCES; i1++) {
    for (i2 = 0; i2 < 3; i2++) {
      if (filterMixCheckBox[i1][i2] == sender()) {
        found = true;
        break;
      }
    }      
    if (found) {
      break;
    }
  }
  // printf("filterMixToggled %d %d\n", i1, i2);
  switch (i2) {
    case 0:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixLfo(i1, filterMixCheckBox[i1][0]->isChecked());
      }
      break;
    case 1:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixRetrigger(i1, filterMixCheckBox[i1][1]->isChecked());
      }
      break;
    case 2:
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setFilterMixHold(i1, filterMixCheckBox[i1][2]->isChecked());
      }
      break;
  }  
}

void Gui::setFilterMod(int value) {

  int i1, i2;
  bool found;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  found = false;   
  for (i1 = 0; i1 < 2; i1++) {
    for (i2 = 0; i2 < 13; i2++) {
      if (filterModSlider[i1][i2] == sender()) {
        found = true;
        break;
      }     
    }
    if (found) {
      break;
    }
  }
//  printf("setFilterMod %d  %d  %d\n", i1, i2, filterModSlider[i1][i2]->getValue());  
  if (i2 < 7) {
    for (int vl = vStart; vl <= vEnd; vl++) {
      val = filterModSlider[i1][i2]->getValue(vl) / 10.0;
      modulationEnvelope e = model->getVoiceData(vl)->getFilterModEnvelope(i1); 
      switch(i2) {
        case 0:
          e.delay = (double)model->getRate() * 0.000001 * pow(val, 3.0);
          break;
        case 1:
          e.attack = (double)STEP/((double)model->getRate() * (1e-5 + 0.000005 * pow(val, 3.0)));
          break;
        case 2:
          e.decay1 = exp(-0.693147 * (double)STEP/(1e-5 + (double)model->getRate() * 0.000005 * pow(val, 3.0)));
          break;
        case 3:
          e.decay2 = exp(-0.693147 * (double)STEP/(1e-5 + (double)model->getRate() * 0.00005 * pow(val, 3.0)));
          break;
        case 4:
          e.level = log(1.0 + (exp(1.0) - 1.0) * val / 1000.0);
          break;
        case 5:
          e.release = exp(-0.693147 * (double)STEP/(1e-5 + (double)model->getRate() * 0.000005 * pow(val, 3.0)));
          break;
        case 6:
          if (e.inverse) {
            e.amount = -1.5 * (pow(10, val / 50.0) - 1.0);
          } else {
            e.amount = 1.5 * (pow(10, val / 50.0) - 1.0); 
          }  
          break;
      }
      model->getVoiceData(vl)->setFilterModEnvelope(i1, e);
    }  
  } else if (i2 < 10) {
    for (int vl = vStart; vl <= vEnd; vl++) {
      val = filterModSlider[i1][i2]->getValue(vl) / 10.0;
      modulationLfo l = model->getVoiceData(vl)->getFilterModLfo(i1); 
      switch(i2) {
        case 7:
          if (l.fast) {
            l.freq = 2.0 * 0.006 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          } else {
            l.freq = 2.0 * 0.0002 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          }  
          break;
        case 8:
          if (l.inverse) {
            l.depth = -1.5 * (pow(10, val / 50.0) - 1.0);
          } else {
            l.depth = 1.5 * (pow(10, val / 50.0) - 1.0);
          }
          break;
        case 9:
          l.delay = (double)model->getRate() * 0.000001 * pow(val, 3.0);
          break;
      }
      model->getVoiceData(vl)->setFilterModLfo(i1, l);  
    }  
  } else {
    for (int vl = vStart; vl <= vEnd; vl++) {
      val = filterModSlider[i1][i2]->getValue(vl) / 10.0;
      modulationLfo l = model->getVoiceData(vl)->getFilterModSpacingLfo(i1); 
      switch(i2) {    
        case 10:
          if (l.fast) {
            l.freq = 2.0 * 0.006 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          } else {
            l.freq = 2.0 * 0.0002 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          }  
          break;
        case 11:
          if (l.inverse) {
            l.depth = -0.03 * (pow(10, val / 50.0) - 1.0);
          } else {
            l.depth = 0.03 * (pow(10, val / 50.0) - 1.0);
          }
          break;
        case 12:
          l.delay = (double)model->getRate() * 0.000001 * pow(val, 3.0);
          break;
      }    
      model->getVoiceData(vl)->setFilterModSpacingLfo(i1, l);
    }
  }
}

void Gui::filterModToggled(int value) {

  int i1, i2;
  bool found, on;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  found = false;
  val = 0;   
  for (i1 = 0; i1 < 2; i1++) {
    for (i2 = 0; i2 < 7; i2++) {
      if (filterModCheckBox[i1][i2] == sender()) {
        found = true;
        break;
      }     
    }
    if (found) {
      break;
    }
  }
  on = filterModCheckBox[i1][i2]->isChecked();
  if (i2 < 1) {
    for (int vl = vStart; vl <= vEnd; vl++) {
      modulationEnvelope e = model->getVoiceData(vl)->getFilterModEnvelope(i1); 
      e.inverse = on;
      val = filterModSlider[i1][6]->getValue(vl) / 10.0;
      if (e.inverse) {
        e.amount = -1.5 * (pow(10, val / 50.0) - 1.0);
      } else {
        e.amount = 1.5 * (pow(10, val / 50.0) - 1.0);
      }
      model->getVoiceData(vl)->setFilterModEnvelope(i1, e);
    }  
  } else if (i2 < 4) {
    for (int vl = vStart; vl <= vEnd; vl++) {
      modulationLfo l = model->getVoiceData(vl)->getFilterModLfo(i1); 
      switch(i2) {
        case 1:
          l.fast = on;
          val = filterModSlider[i1][7]->getValue(vl) / 10.0;
          if (l.fast) {
            l.freq = 2.0 * 0.006 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          } else {
            l.freq = 2.0 * 0.0002 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          }  
          break;
        case 2:
          l.retrigger = on;
          break;
        case 3:
          l.inverse = on;
          val = filterModSlider[i1][8]->getValue(vl) / 10.0;
          if (l.inverse) {
            l.depth = -1.5 * (pow(10, val / 50.0) - 1.0);
          } else {
            l.depth = 1.5 * (pow(10, val / 50.0) - 1.0);
          }
          break;
      }
      model->getVoiceData(vl)->setFilterModLfo(i1, l);  
    }  
  } else {
    for (int vl = vStart; vl <= vEnd; vl++) {
      modulationLfo l = model->getVoiceData(vl)->getFilterModSpacingLfo(i1); 
      switch(i2) {    
        case 4:
          l.fast = on;
          val = filterModSlider[i1][10]->getValue(vl) / 10.0;
          if (l.fast) {
            l.freq = 2.0 * 0.006 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          } else {
            l.freq = 2.0 * 0.0002 * pow(val, 2.0) * (double)STEP / (double)model->getRate();
          }  
          break;
        case 5:
          l.retrigger = on;
          break;
        case 6:
          l.inverse = on;
          val = filterModSlider[i1][11]->getValue(vl) / 10.0;
          if (l.inverse) {
            l.depth = -0.03 * (pow(10, val / 50.0) - 1.0);
          } else {
            l.depth = 0.03 * (pow(10, val / 50.0) - 1.0);
          }
          break;
      }    
      model->getVoiceData(vl)->setFilterModSpacingLfo(i1, l);
    }  
  }  
//  printf("setFilterModToggled %d  %d  %d\n", i1, i2, filterModCheckBox[i1][i2]->isChecked());  
}

void Gui::setModulation(int filterIndex, int paramIndex, int voiceIndex, double modValue, int value) {


  double c;
 
  if (paramIndex < 13) {
    c = (filterModSlider[filterIndex][paramIndex]->maximum() - filterModSlider[filterIndex][paramIndex]->minimum()) / 127.0;
    filterModSlider[filterIndex][paramIndex]->setValues(voiceIndex, modValue, filterModSlider[filterIndex][paramIndex]->minimum() + (int)(c * (double)value));
  } else {
    filterModCheckBox[filterIndex][paramIndex-13]->setChecked(value > 62);
  }
//  printf("Modulation %d  %d  %d\n", filterIndex, paramIndex, value); 
}

void Gui::setMasterVolume(int value) {
  
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (int vl = vStart; vl <= vEnd; vl++) {
    val = ((ModSlider *)sender())->getValue(vl);
    model->getVoiceData(vl)->setMasterVolume(32.0 * (pow(10, (double)val / 500.0) - 1.0));
  }
}

void Gui::setNoise(int value) {

  int i;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  for (i = 0; i < 4; i++) {
    if (noiseSlider[i] == sender()) {
      switch (i) {
        case 0:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setDelta(4, 12.0 * (val / 500.0 - 1.0));
          }  
          break;
        case 1:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setDelta(5, 5.0 + 4.0 * (val / 500.0 - 1.0));
          }
          break;              
        case 2:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setEvenOdd(3, val / 1000.0);
          }
          break;              
        case 3:
          for (int vl = vStart; vl <= vEnd; vl++) {
            val = ((ModSlider *)sender())->getValue(vl);
            model->getVoiceData(vl)->setNoiseBandwidth(0.25 + 0.24 * (val / 500.0 - 1.0));
          }
          break;              
      }
      break;
    }  
  }   
}

void Gui::setNoise(int objectType, int objectIndex, int voiceIndex, double modValue, int value) {

  double c;

  switch (objectIndex) {
    case 0:
      c = (noiseSlider[0]->maximum() - noiseSlider[0]->minimum()) / 127.0;
      noiseSlider[0]->setValues(voiceIndex, modValue, noiseSlider[0]->minimum() + (int)(c * (double)value));
      break;
    case 1:
      c = (noiseSlider[1]->maximum() - noiseSlider[1]->minimum()) / 127.0;
      noiseSlider[1]->setValues(voiceIndex, modValue, noiseSlider[1]->minimum() + (int)(c * (double)value));
      break;
    case 2:
      c = (noiseSlider[2]->maximum() - noiseSlider[2]->minimum()) / 127.0;
      noiseSlider[2]->setValues(voiceIndex, modValue, noiseSlider[2]->minimum() + (int)(c * (double)value));
      break;
    case 3:
      c = (noiseSlider[3]->maximum() - noiseSlider[3]->minimum()) / 127.0;
      noiseSlider[3]->setValues(voiceIndex, modValue, noiseSlider[3]->minimum() + (int)(c * (double)value));
      break;
  }
}

void Gui::setModDepth(int value) {

  // printf("setModDepth %d\n", value);
  if (!midiControllerTree->topLevelItemCount() || (midiControllerTree->currentItem() == NULL)) {
    modDepthLabel->setDisabled(true);
    modDepthSlider->setDisabled(true);
    modFullToggle->setDisabled(true);
    modInverseToggle->setDisabled(true);
    return;
  }
  QTreeWidgetItem *m = midiControllerTree->currentItem();
  model->setControllerModulation(m->parent(), m, value, modFullToggle->isChecked(), modInverseToggle->isChecked());
}

void Gui::modFullToggled(int value) {

  // printf("modFullToggled %d\n", value);
  modDepthLabel->setDisabled(value);
  modDepthSlider->setDisabled(value);
  modInverseToggle->setDisabled(value); // TODO Inverse auch für Full Mode
  if (!midiControllerTree->topLevelItemCount() || (midiControllerTree->currentItem() == NULL)) {
    modFullToggle->setDisabled(true);
    return;
  }
  QTreeWidgetItem *m = midiControllerTree->currentItem();
  model->setControllerModulation(m->parent(), m, modDepthSlider->value(), modFullToggle->isChecked(), modInverseToggle->isChecked());
}

void Gui::modInverseToggled(int value) {

  // printf("modInverseToggled %d\n", value);
  if (!midiControllerTree->topLevelItemCount() || (midiControllerTree->currentItem() == NULL)) {
    modDepthLabel->setDisabled(true);
    modDepthSlider->setDisabled(true);
    modFullToggle->setDisabled(true);
    modInverseToggle->setDisabled(true);
    return;
  }
  QTreeWidgetItem *m = midiControllerTree->currentItem();
  model->setControllerModulation(m->parent(), m, modDepthSlider->value(), modFullToggle->isChecked(), modInverseToggle->isChecked());
}

void Gui::midiCurrentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *previous) {

  controlObject co;
  bool success;

  // printf("midiCurrentItemChanged\n");
  if (!midiControllerTree->topLevelItemCount() || (midiControllerTree->currentItem() == NULL)) {
    modDepthLabel->setDisabled(true);
    modDepthSlider->setDisabled(true);
    modFullToggle->setDisabled(true);
    modInverseToggle->setDisabled(true);
    return;
  }
  co = model->getControlObject(current->parent(), current, success); 
  if (success) {
    modDepthLabel->setEnabled(true);
    modDepthSlider->setEnabled(true);
    modFullToggle->setEnabled(true);
    modInverseToggle->setEnabled(true);
    modDepthSlider->setValue(co.modDepth);
    modFullToggle->setChecked(co.full);
    modInverseToggle->setChecked(co.inverse);
  } else {
    modDepthLabel->setDisabled(true);
    modDepthSlider->setDisabled(true);
    modFullToggle->setDisabled(true);
    modInverseToggle->setDisabled(true);
  }
}

void Gui::setCompressor(int value) {

  int i1;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);    
  for (i1 = 0; i1 < 4; i1++) {
    if (compressorSlider[i1] == sender()) {
      for (int vl = vStart; vl <= vEnd; vl++) {
        val = ((ModSlider *)sender())->getValue(vl);
        model->getVoiceData(vl)->setCompressor(i1, val);
      }
      break;
    }
  }  
}

void Gui::voiceRangeSet() {

  int i1, vStart, vEnd, old_vStart, old_vEnd, currentIndex;
  QString qs;
  VoiceData *vd;

  vStart = voiceRangeStart->value() - 1;
  vEnd = voiceRangeEnd->value() - 1;
  model->setEditRange(vStart, vEnd);
  for (i1 = vStart; i1 <= vEnd; i1++) {
    vd = model->getVoiceData(i1);
 //   vd->setFileName(voiceFileName); // TODO may later load files here as well 
    vd->setChannel(voiceChannel->value() - 1);
    vd->setEditRange(vStart, vEnd);
//    synth->voiceSynth[i1]->loadVoiceData(voiceRangeStart->value() - 1); // TODO check this
  }
  if (vStart > 0) {
    vd = model->getVoiceData(vStart - 1);
    vd->getEditRange(old_vStart, old_vEnd);
    for (i1 = old_vStart; i1 < vStart; i1++) {
      vd = model->getVoiceData(i1);
      vd->setEditRange(old_vStart, vStart - 1);
    }
  }  
  if (vEnd < model->getPoly() - 1) {
    vd = model->getVoiceData(vEnd + 1);
    vd->getEditRange(old_vStart, old_vEnd);
    for (i1 = vEnd + 1; i1 < old_vEnd; i1++) {
      vd = model->getVoiceData(i1);
      vd->setEditRange(vEnd + 1, old_vEnd);
    }
  }  
  currentIndex = voiceTree->indexOfTopLevelItem(voiceTree->currentItem());
  if (currentIndex < 0) {
    currentIndex = vStart;
  }
  voiceTree->clear();
  for (i1 = 0; i1 < model->getPoly(); i1++) {
    vd = model->getVoiceData(i1);
    QTreeWidgetItem *item = new QTreeWidgetItem(voiceTree);
    qs = QString::number(i1 + 1);
    item->setText(0, qs);
    qs = QString::number(vd->getChannel() + 1);
    item->setText(1, qs);
    item->setText(2, vd->getFileName());
    if (i1 == currentIndex) {
      voiceTree->setCurrentItem(item);
    }
  }  
}

void Gui::voiceRangeStartChanged(int value) {


  //printf("voiceRangeStartChanged: %d\n", value);
  if (value > voiceRangeEnd->value()) {
    voiceRangeEnd->setValue(value);
  }
}

void Gui::voiceRangeEndChanged(int value) {

  //printf("voiceRangeEndChanged: %d\n", value);
  if (value < voiceRangeStart->value()) {
    voiceRangeStart->setValue(value);
  }
}

void Gui::voiceChannelChanged(int value) { 

  //printf("voiceChannelChanged: %d\n", value);
}

void Gui::saveVoice(int voiceIndex) {

  int i1, i2, i3;
  int d1, d2;
  QString fn, qs;
  FILE *f;
  int vStart, vEnd;
  VoiceData *vd;

  modifiedFlag = false;
  vd = model->getVoiceData(voiceIndex);
  vd->getEditRange(vStart, vEnd);
  fn = vd->getFileName();
  if (!fn.isEmpty()) {
    if ((f = fopen(fn.toLatin1(), "w"))) {
      voiceFileName = fn;
      voiceFileLabel->setText(fn);
      fprintf(f, "%d\n", masterVolume->value());
      fprintf(f, "%d\n", pitchLFOfreq->value());
      fprintf(f, "%d\n", pitchLFOdepth->value());
      fprintf(f, "%d\n", morphingSlider->value());
      for (i1 = 0; i1 < 4; i1++) {
        fprintf(f, "%d\n", compressorSlider[i1]->value());
      }
      for (i1 = 0; i1 < 2; i1++) {
        fprintf(f, "%d\n", panSlider[i1]->value());
      }
      fprintf(f, "%d\n", panMode->currentIndex());
      for (i1 = 0; i1 < 3; i1++) {
        fprintf(f, "%d\n", evenOddSlider[i1]->value());
      }
      for (i1 = 0; i1 < 4; i1++) {
        fprintf(f, "%d\n", noiseSlider[i1]->value());
      }
      for (i1 = 0; i1 < MAX_OSC - 1; i1++) {
        fprintf(f, "%d\n", spectrumOffset[i1]->value());
        fprintf(f, "%d\n", spectrumSpacing[i1]->value());
      }  
      for (i1 = 0; i1 < 2; i1++) {
        fprintf(f, "%d\n", spectrumOffsetCombo[i1]->currentIndex());
        fprintf(f, "%d\n", spectrumSpacingCombo[i1]->currentIndex());
      }
      for (i1 = 0; i1 < MAX_SPLIT; i1++) {
        fprintf(f, "%d\n", model->getHarmonicSplit(i1)); 
      }  
      for (i1 = 0; i1 < MAX_PARAMS; i1++) {
        for (i2 = 0; i2 < MAX_SCENES * MAX_SOURCES; i2++) {
          harmonicScale[i1]->readBufValue(i2, d1, d2);
          fprintf(f, "%d\n", d1);
        }  
        for (i2 = 0; i2 < model->getNumHarmonics(); i2++) {
          fprintf(f, "%f\n", model->getVoiceData(vStart)->getHarmonic(i1, i2)); 
        }
      }
      for (i1 = 0; i1 < MAX_SOURCES; i1++) {
        for (i2 = 0; i2 < 6; i2++) {
          fprintf(f, "%d\n", filterMixSlider[i1][i2]->value());
        }
        for (i2 = 0; i2 < 3; i2++) {
          fprintf(f, "%d\n", filterMixCheckBox[i1][i2]->isChecked() ? 1 : 0);
        }
      }                  
      for (i1 = 0; i1 < 2; i1++) {
        for (i3 = 0; i3 < 2; i3++) {
          fprintf(f, "%d\n", filterCutoff[2 * i1 + i3]->value());
          fprintf(f, "%d\n", filterSpacing[2 * i1 + i3]->value());
        }
        fprintf(f, "%d\n", filterKeyTracking[i1]->value());
        fprintf(f, "%d\n", filterDepth[i1]->value());
        fprintf(f, "%d\n", filterCutoffCombo[i1]->currentIndex());
        fprintf(f, "%d\n", filterSpacingCombo[i1]->currentIndex());
        fprintf(f, "%d\n", filterTrackingCombo[i1]->currentIndex());
        for (i2 = 0; i2 < FILTER_LEN; i2++) {
          fprintf(f, "%f\n", model->getVoiceData(vStart)->getFilter(i1, i2)); 
        }
      }
      for (i1 = 0; i1 < 2; i1++) {
        for (i2 = 0; i2 < 13; i2++) {
          fprintf(f, "%d\n", filterModSlider[i1][i2]->value());
        }
        for (i2 = 0; i2 < 7; i2++) {
          fprintf(f, "%d\n", filterModCheckBox[i1][i2]->isChecked() ? 1 : 0);
        }
      }                  
      fclose(f);
    }  
  }   
}

void Gui::editVoice(int voiceIndex) {

  int vStart, vEnd;
  VoiceData *vd;

  modifiedFlag = false;
  vd = model->getVoiceData(voiceIndex);
  vd->getEditRange(vStart, vEnd);
  model->setEditRange(vStart, vEnd);
  voiceChannel->setValue(vd->getChannel() + 1);
  voiceRangeStart->setValue(vStart+1);
  voiceRangeEnd->setValue(vEnd+1);
  loadVoice(voiceIndex);
}                

void Gui::loadVoice(int voiceIndex) {

  int i1, i2, i3;
  int vStart, vEnd;
  QString fn, qs;
  char buf[256]; 
  QFile f;
  VoiceData *vd;

  modifiedFlag = false;
  model->getEditRange(vStart, vEnd);
  vd = model->getVoiceData(voiceIndex);
  fn = vd->getFileName();
  if (!fn.isEmpty()) {
    f.setFileName(fn);
    voiceFileLabel->setText(fn);
    if (f.exists()) {
      f.open(QIODevice::ReadOnly);
      f.readLine(buf, sizeof(buf));
      qs = QString(buf);
      masterVolume->setValue(qs.toInt());
      f.readLine(buf, sizeof(buf));
      qs = QString(buf);
      pitchLFOfreq->setValue(qs.toInt());
      f.readLine(buf, sizeof(buf));
      qs = QString(buf);
      pitchLFOdepth->setValue(qs.toInt());
      f.readLine(buf, sizeof(buf));
      qs = QString(buf);
      morphingSlider->setValue(qs.toInt());
      for (i1 = 0; i1 < 4; i1++) {
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        compressorSlider[i1]->setValue(qs.toInt());
      }
      for (i1 = 0; i1 < 2; i1++) {
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        panSlider[i1]->setValue(qs.toInt());
      }
      f.readLine(buf, sizeof(buf));
      qs = QString(buf);
      panMode->setCurrentIndex(qs.toInt());
      for (i1 = 0; i1 < 3; i1++) {
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        evenOddSlider[i1]->setValue(qs.toInt());
      }
      for (i1 = 0; i1 < 4; i1++) {
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        noiseSlider[i1]->setValue(qs.toInt());
      }
      for (i1 = 0; i1 < MAX_OSC - 1; i1++) {
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        spectrumOffset[i1]->setValue(qs.toInt());
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        spectrumSpacing[i1]->setValue(qs.toInt());
      }
      for (i1 = 0; i1 < 2; i1++) {  
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        spectrumOffsetCombo[i1]->setCurrentIndex(qs.toInt());
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        spectrumSpacingCombo[i1]->setCurrentIndex(qs.toInt());
      }
      for (i1 = 0; i1 < MAX_SPLIT; i1++) {
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        model->setHarmonicSplit(i1, qs.toInt());
//        model->currentVoiceData()->setHarmonicSplit(i1, qs.toInt()); // TODO Check if harmonicSplit should be updated in VoiceData
      }
      for (i1 = 0; i1 < MAX_PARAMS; i1++) {
        for (i2 = 0; i2 < MAX_SCENES * MAX_SOURCES; i2++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          harmonicScale[i1]->writeBufValue(i2, qs.toInt(), 0);
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setGroupScale(i2, i1, qs.toInt());
          }  
        }
        for (i2 = 0; i2 < model->getNumHarmonics(); i2++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setHarmonic(i1, i2, qs.toFloat());
          }  
        }
      }
      for (i1 = 0; i1 < MAX_SOURCES; i1++) {
        for (i2 = 0; i2 < 6; i2++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          filterMixSlider[i1][i2]->setValue(qs.toInt());
        }
        for (i2 = 0; i2 < 3; i2++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          filterMixCheckBox[i1][i2]->setChecked(qs.toInt());
        }
      }   
      for (i1 = 0; i1 < 2; i1++) {
        for (i3 = 0; i3 < 2; i3++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          filterCutoff[2 * i1 + i3]->setValue(qs.toInt());
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          filterSpacing[2 * i1 + i3]->setValue(qs.toInt());
        }
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        filterKeyTracking[i1]->setValue(qs.toInt());        
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        filterDepth[i1]->setValue(qs.toInt());
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        filterCutoffCombo[i1]->setCurrentIndex(qs.toInt());
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        filterSpacingCombo[i1]->setCurrentIndex(qs.toInt());
        f.readLine(buf, sizeof(buf));
        qs = QString(buf);
        filterTrackingCombo[i1]->setCurrentIndex(qs.toInt());
        for (i2 = 0; i2 < FILTER_LEN; i2++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          for (int vl = vStart; vl <= vEnd; vl++) {
            model->getVoiceData(vl)->setFilter(i1, i2, qs.toFloat());
          }
        }
      }
      for (i1 = 0; i1 < 2; i1++) {
        for (i2 = 0; i2 < 13; i2++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          filterModSlider[i1][i2]->setValue(qs.toInt());
        }
        for (i2 = 0; i2 < 7; i2++) {
          f.readLine(buf, sizeof(buf));
          qs = QString(buf);
          filterModCheckBox[i1][i2]->setChecked(qs.toInt());
        }
      }                  
    }
  }
  groupSelector->button(0)->setChecked(true);
  selectGroup(0);
  harmonicTab->setCurrentIndex(0);
  updateTab(0);
  model->resetLFO();
}                

void Gui::voiceTreeChanged(QTreeWidgetItem *item, QTreeWidgetItem *previous) {

  int i1, i2;
  QString qs;
  int vStart, vEnd;
  VoiceData *vd;  
  QMessageBox::StandardButton sb;
  
  i1 = voiceTree->indexOfTopLevelItem(previous);
  i2 = voiceTree->indexOfTopLevelItem(item);
  if ((i1 < 0) || (i2 < 0)) {
    return;
  } else {
    vd = model->getVoiceData(i1);
    vd->getEditRange(vStart, vEnd);
    if ((i2 < vStart) || (i2 > vEnd)) {
      if (modifiedFlag) {
        qs = "Save "+vd->getFileName()+" ?";
  //      printf("previous: %d   index: %d\n", i1, i2);
        sb = QMessageBox::question(this, "Save Voice", qs, QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes); 
        if (sb == QMessageBox::Yes) {
          saveVoice(i1);
        }
      }  
      editVoice(i2);
    }  
  }  
}

void Gui::mainTabChanged(int index) {

  if (index != 10) {
    modifiedFlag = true;
  }  
}

void Gui::setPan(int value) {

  int i1;
  double val;
  int vStart, vEnd;
  
//  printf("setPan: %d\n", value);
  model->getEditRange(vStart, vEnd);    
  for (i1 = 0; i1 < 4; i1++) {
    if (panSlider[i1] == sender()) {
      for (int vl = vStart; vl <= vEnd; vl++) {
        val = ((ModSlider *)sender())->getValue(vl);
        model->getVoiceData(vl)->setPan(i1, val);
      }
      break;
    }
  }  
}

void Gui::setPanMode(int index) {

  int vStart, vEnd;

//  printf("PanMode: %d\n", index);
  model->getEditRange(vStart, vEnd);    
  for (int vl = vStart; vl <= vEnd; vl++) {
    model->getVoiceData(vl)->setPan(2, index);
  }  
}
