#ifndef GUI_H
#define GUI_H

#include <stdio.h>
#include <stdlib.h>
#include <QtGui/QWidget>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QTabWidget>
#include <QtGui/QButtonGroup>
#include <QtGui/QSlider>
#include <QtGui/QLabel>
#include <QtGui/QCheckBox>
#include <QtGui/QComboBox>
#include <QtGui/QTreeWidget>
#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>
#include <QtGui/QSpinBox>
#include "model.h"
#include "synth.h"
#include "multislider.h"
#include "modslider.h"
#include "harmonics.h"
#include "filter.h"
#include "jackprocessor.h"

class Gui : public QWidget
{
  Q_OBJECT

  private:
    Model *model;
    JackProcessor *jackProcessor;
    Synth *synth;
    Harmonics *harmonics[MAX_PARAMS];
    Filter *filter[2];
    MultiSlider *harmonicScale[MAX_PARAMS];
    ModSlider *pitchLFOfreq, *pitchLFOdepth, *morphingSlider, *masterVolume;
    ModSlider *spectrumOffset[4], *spectrumSpacing[4], *evenOddSlider[3];
    ModSlider *filterCutoff[4], *filterSpacing[4], *filterKeyTracking[2], *filterDepth[2];
    ModSlider *filterMixSlider[MAX_SOURCES][6];
    QCheckBox *filterMixCheckBox[MAX_SOURCES][3];
    ModSlider *filterModSlider[2][13];
    QCheckBox *filterModCheckBox[2][7];
    ModSlider *noiseSlider[4];
    ModSlider *compressorSlider[4];
    ModSlider *panSlider[2];
    QComboBox *panMode; 
    QComboBox *spectrumOffsetCombo[2], *spectrumSpacingCombo[2];
    QComboBox *filterCutoffCombo[2], *filterSpacingCombo[2], *filterTrackingCombo[2];
    QSlider *modDepthSlider;
    QLabel *modDepthLabel;
    QCheckBox *modFullToggle, *modInverseToggle;
    QTabWidget *mainTab;
    QTabWidget *harmonicTab;
    QButtonGroup *groupSelector;
    QTreeWidget *midiControllerTree, *controlObjectTree;
    QLabel *voiceFileLabel;
    QString voiceFileName;
    QPushButton *voiceRangeSetButton;
    QSpinBox *voiceRangeStart, *voiceRangeEnd, *voiceChannel;
    QTreeWidget *voiceTree;
    bool followMidi, modifiedFlag;
    void addControlFolder(QString name);
    void addControlObject(QString name, int objectType, int objectIndex);
    void setFilter(int index, int voiceIndex, double modValue, int value);
    void setMixer(int index, int voiceIndex, double modValue, int value);
    void setModulation(int filterIndex, int paramIndex, int voiceIndex, double modValue, int value);

  public:
    Gui(Model *p_model, QWidget* parent=0);
    ~Gui();
 
  public slots: 
    void editVoice(int voiceIndex);
    void loadVoice(int voiceIndex);
    void saveVoice(int voiceIndex);
    void mainTabChanged(int index);
    void voiceTreeChanged(QTreeWidgetItem *item, QTreeWidgetItem *previous);
    void loadAll();
    void saveAll();
    void loadMidi();
    void saveMidi();
    void reScale(int value);
    void selectGroup(int p_group);
    void updateTab(int id);
    void closeGui();
    void setCommon(int objectType, int objectIndex, int voiceIndex, double modValue, int value);
    void setOscillators(int objectType, int objectIndex, int voiceIndex, double modValue, int value);
    void setGlobalScale(int objectType, int objectIndex, int voiceIndex, double modValue, int value);
    void setPitchLFOfreq(int value);
    void setPitchLFOdepth(int value);
    void setEvenOdd(int value);
    void setMasterVolume(int value);
    void setSpectrumOffset(int value);
    void setSpectrumSpacing(int value);
    void setSpectrumOffsetCombo(int index);
    void setSpectrumSpacingCombo(int index);
    void setMorphing(int value);
    void setCompressor(int value);
    void setPan(int value);
    void setPanMode(int index);
    void setFilterCutoff(int value);
    void setFilterSpacing(int value);
    void setFilterCutoffCombo(int index);
    void setFilterSpacingCombo(int index);
    void setFilterKeyTracking(int value); 
    void setFilterTrackingCombo(int value); 
    void setFilterDepth(int value);
    void setFilterMix(int value);
    void filterMixToggled(int value);
    void setFilterMod(int value);
    void filterModToggled(int value);
    void setNoise(int value);
    void setNoise(int objectType, int objectIndex, int voiceIndex, double modValue, int value);
    void setModDepth(int value);
    void modFullToggled(int value);
    void modInverseToggled(int value);
    void connectController();
    void connectController(QTreeWidgetItem *midiControllerItem, int objectType, int objectIndex, int modDepth, bool full, bool inverse);
    void disconnectController();
    void processMidiController(int type, unsigned char eventChannel,
                               unsigned int eventParam, int value, int voiceIndex);
    bool hasMidiController(int type, unsigned char eventChannel,
                           unsigned int eventParam, int value, int voiceIndex);
    void addMidiController(int type, unsigned char eventChannel,
                           unsigned int eventParam); 
    void midiCurrentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *previous);
    void voiceRangeSet();
    void voiceRangeStartChanged(int value);
    void voiceRangeEndChanged(int value);
    void voiceChannelChanged(int value);
};

#endif
