#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <QtGui/QWidget>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QPainter>
#include <QtGui/QMouseEvent>
#include <QtCore/QString>
#include "model.h"
#include "harmonics.h"

Harmonics::Harmonics(Model *p_model, QString p_unitLabel, int p_index, QWidget *parent, int w, int h) : QWidget(parent) {

  model = p_model;
  unitLabel = p_unitLabel;
  index = p_index;
  mousePos = -1;
  mouseVal = 0;
  setMinimumSize(w, h);
  leftBorder = 20;
  rightBorder = 10;
  topBorder = 20;
  bottomBorder = 60;
  scaleX = (double)(width() - leftBorder - rightBorder) / (double)model->getHarmonicSplit(0);
  scaleY = (height() - topBorder - bottomBorder) / model->getMax();
  setPalette(QPalette(QColor(200, 200, 200), QColor(20, 0, 70)));
  setAutoFillBackground(true);
  setMouseTracking(true);
}

Harmonics::~Harmonics() {
  
}

void Harmonics::paintEvent(QPaintEvent *) {

  QPainter p;
  QString qs;
  int i, o, n; // TODO read data from VoiceData throughout this method
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  o = model->getHarmonicSplit(model->getEditGroup());
  n = model->getHarmonicSplit(model->getEditGroup() + 1) - o;
  scaleX = (double)(width() - leftBorder - rightBorder) / (double)n;
  scaleY = (height() - topBorder - bottomBorder) / model->getMax();
  p.begin(this);
  p.setPen(QPen(QBrush(QColor(175, 131, 0)), 10));
  for (i=0; i < n; i++) {  
    p.drawLine(leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder - scaleY * model->getVoiceData(vStart)->getHarmonic(index, i + o), 
               leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder); 
  }
  p.setPen(QPen(QBrush(QColor(194, 146, 0)), 8));
  for (i=0; i < n; i++) {  
    p.drawLine(leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder - scaleY * model->getVoiceData(vStart)->getHarmonic(index, i + o), 
               leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder); 
  }
  p.setPen(QPen(QBrush(QColor(216, 162, 0)), 6));
  for (i=0; i < n; i++) {  
    p.drawLine(leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder - scaleY * model->getVoiceData(vStart)->getHarmonic(index, i + o), 
               leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder); 
  }
  p.setPen(QPen(QBrush(QColor(240, 180, 0)), 4));
  for (i=0; i < n; i++) {  
    p.drawLine(leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder - scaleY * model->getVoiceData(vStart)->getHarmonic(index, i + o), 
               leftBorder + (int)((double)i*scaleX), 
               height() - bottomBorder); 
  }
  if ((mousePos > -1) && (mousePos < n) && (mouseVal >= 0) && (mouseVal <= model->getMax())) {
    p.setFont(QFont("Helvetica", 14)); 
    qs = "Harmonic " + QString::number(mousePos + 1) + " :  "  
       + QString::number(model->getVoiceData(vStart)->getHarmonic(index, mousePos + o), 'f', 1) + unitLabel 
       + " <-- " + QString::number(mouseVal, 'f', 1) + unitLabel;
    p.drawText(leftBorder, height() - bottomBorder * 0.3, qs);
  }  
  p.end();
}

void Harmonics::viewportResizeEvent(QResizeEvent *ev) {

  scaleX = (double)(width() - leftBorder - rightBorder) / (double)model->getNumHarmonics();
  scaleY = (height() - topBorder - bottomBorder) / model->getMax();
  update(0, 0, width(), height());
}

void Harmonics::mousePressEvent (QMouseEvent *ev) {

  int i, o, n;
  double val;
  int vStart, vEnd;
  
  model->getEditRange(vStart, vEnd);
  i = round((ev->x() - leftBorder) / scaleX);
  o = model->getHarmonicSplit(model->getEditGroup());
  n = model->getHarmonicSplit(model->getEditGroup() + 1) - o;
  if ((i >= 0) && (i < n)) { 
    val = (double)(height() - bottomBorder - ev->y()) / scaleY;
    for (int vl = vStart; vl <= vEnd; vl++) {
      model->getVoiceData(vl)->setHarmonic(index, i + o, val);  
    }
    update(0, 0, width(), height());
  }
}

void Harmonics::mouseReleaseEvent (QMouseEvent *ev) {

  update(0, 0, width(), height());
}

void Harmonics::mouseMoveEvent (QMouseEvent *ev) {

  int o, n;
  int vStart, vEnd;

  model->getEditRange(vStart, vEnd);
  mousePos = round((ev->x() - leftBorder) / scaleX);
  o = model->getHarmonicSplit(model->getEditGroup());
  n = model->getHarmonicSplit(model->getEditGroup() + 1) - o;
  if ((mousePos >= 0) && (mousePos < n)) {
    mouseVal = (double)(height() - bottomBorder - ev->y()) / scaleY;
    if (ev->buttons() == Qt::LeftButton) {
      for (int vl = vStart; vl <= vEnd; vl++) {
        model->getVoiceData(vl)->setHarmonic(index, mousePos + o, mouseVal);  
      }
      update(0, 0, width(), height());
    } else { 
      update(0, height() - bottomBorder, width(), height());
    } 
  }   
}
                  