#include <stdio.h>      
#include <stdlib.h>     
#include <unistd.h>
#include <getopt.h>  
#include <QtGui/QApplication>
#include <QtGui/QMainWindow>
#include <QtGui/QMenuBar>   
#include <QtGui/QMenu>   
#include <QtGui/QInputDialog>   
#include <QtCore/QString>   
#include "model.h"
#include "gui.h"
#include "jackprocessor.h"

static struct option options[] =
        {{"poly", 1, 0, 'p'}, 
         {"verbose", 0, 0, 'v'},  
         {"help", 0, 0, 'h'},  
         {0, 0, 0, 0}};

int main(int argc, char *argv[])  
{
  int getopt_return;
  int option_index;
  bool verbose, ok, set_poly;
  int poly, rate, bufsize, i;    

  poly = 4;
  rate = 44100;
  bufsize = 1024;
  verbose = false;
  set_poly = true;
  QApplication *myApp = new QApplication(argc, argv);
  while((getopt_return = getopt_long(argc, argv, "vhp:", options, &option_index)) >= 0) {
    switch(getopt_return) {
      case 'p':
        poly = atoi(optarg);
        if (poly < 1) poly = 1; 
        if (poly > MAX_POLY) poly = MAX_POLY;
        set_poly = false;
        break;
      case 'v':
        verbose = true;
        break;
      case 'h': 
        printf("\n\nAdd64     (c)2011 by Matthias Nagorni\n\n");
        printf("--poly            number of voices [1]\n");
//        printf("--verbose         print underrun messages\n");
        printf("--help            print this help page\n\n");
        exit(EXIT_SUCCESS);
    }
  }                          
  QMainWindow *top = new QMainWindow();
  top->setWindowTitle("Add64    (c)2011 by Matthias Nagorni");
  if (set_poly) {
    i = QInputDialog::getInt(top, "Add64 Polyphony", "Polyphony: ", 4, 1, MAX_POLY, 1, &ok);
    if (ok && (i > 0) && (i <= MAX_POLY)) {
      poly = i;
    } 
  }   
  Model *model = new Model(poly);
  model->setVerbose(verbose);
  model->setRate(rate);
  model->setBufSize(bufsize);
  model->setEditRange(0, poly - 1);
  Gui *gui = new Gui(model, top);
  QMenu *fileMenu = top->menuBar()->addMenu("&File");
  fileMenu->addAction("&Load Sound", gui, SLOT(loadAll()));
  fileMenu->addAction("&Save Sound", gui, SLOT(saveAll()));
  fileMenu->addSeparator();
  fileMenu->addAction("&Load MIDI Map", gui, SLOT(loadMidi()));
  fileMenu->addAction("&Save MIDI Map", gui, SLOT(saveMidi()));
  fileMenu->addSeparator();
  fileMenu->addAction("&Quit", myApp, SLOT(quit()));
  QObject::connect(myApp, SIGNAL(aboutToQuit()), gui, SLOT(closeGui())); 
  top->setCentralWidget(gui); 
  top->show();
  return myApp->exec(); 
}
