#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <QtCore/QObject>
#include <QtGui/QTreeWidgetItem>
#include "voicedata.h"
#include "model.h"

Model::Model(int p_poly, QObject *parent) : QObject(parent) {

  int i1, i2;

  rate = 44100;
  max = 100.0;
  editStart = 0;
  editEnd = 0;
  verbose = true;
  harmonicSplit[0] = 0;
  harmonicSplit[1] = 64; 
  harmonicSplit[2] = 96;
  harmonicSplit[3] = 128; 
  harmonicSplit[4] = 136;
  harmonicSplit[5] = 200;
  harmonicSplit[6] = 232;
  harmonicSplit[7] = 264;
  harmonicSplit[8] = 272;
  delta[0] = 1.5;
  delta[1] = 0.7;
  delta[2] = 1.5;
  delta[3] = 0.7;
  delta[4] = 1.0;
  delta[5] = 1.0;
  numHarmonics = harmonicSplit[MAX_SPLIT - 1];
  morphing = 0.5;
  linearLevel = 0.5;
  transitionWidth = 0.5;
  compression = 0.5;
  antialiasing = 4.0;
  poly = p_poly;
  masterVolume = 4e4;
  pitchBend = 0;
  noiseBandwidth = 0.25;
  voices = (voiceType *)malloc(poly * sizeof(voiceType));
  for (i1 = 0; i1 < 2; i1++) {
    buf[i1] = (double *)malloc(MAX_FRAMES * sizeof(double));
    memset(buf[i1], 0, MAX_FRAMES * sizeof(double));
  }                                     
  currentVoiceDataIndex = 0;
  for (i1 = 0; i1 < poly; i1++) {
    voiceData[i1] = new VoiceData();
    voiceData[i1]->setEditRange(0, poly);
    voiceData[i1]->setRate(rate);
    voiceData[i1]->setVoice(&voices[i1]);
    voices[i1].note = 0;
    voices[i1].velocity = 0;
    voices[i1].gate = false;
    voices[i1].active = false;
    voices[i1].samples = 1e9;
    voices[i1].cut = 20000.0;
    for (i2 = 0; i2 < MAX_SOURCES; i2++) {
      voices[i1].filterMixEnv[i2] = 0;
      voices[i1].filterMixEnvState[i2] = 0;
      voices[i1].filterMixSampleStack[i2] = 0;
    }  
    for (i2 = 0; i2 < 2; i2++) {
      voices[i1].filterModEnv[i2] = 0;
      voices[i1].filterModEnvState[i2] = 0;
      voices[i1].filterModLfo[i2] = 0;
      voices[i1].filterModLfoState[i2] = 0;
      voices[i1].filterModSpacingLfo[i2] = 0;
      voices[i1].filterModSpacingLfoState[i2] = 0;
    }
    envelopes[i1] = (envelopeType *)malloc(numHarmonics * sizeof(envelopeType));
    for (i2 = 0; i2 < numHarmonics; i2++) {
      envelopes[i1][i2].val = 0;
      envelopes[i1][i2].state = 0;
      envelopes[i1][i2].lfo = 0;
      envelopes[i1][i2].lfo_state = 0;
    }  
  }
  for (i2 = 0; i2 < MAX_SOURCES; i2++) {
    filterMixLfo[i2] = 0;
    filterMixRetrigger[i2] = 0;
    filterMixHold[i2] = 0;
  }                
  bufSize = 1024;
  scaleLin[0] = 1.0;
  scaleExp[0] = 2.0;
  minScale[0] = 0.1;
  maxScale[0] = 10.0;
  scaleLin[1] = 0.000001;
  scaleExp[1] = 3.0;
  minScale[1] = 0.1;
  maxScale[1] = 10.0;
  scaleLin[2] = 0.000001;
  scaleExp[2] = 3.0;
  minScale[2] = 0.1;
  maxScale[2] = 10.0;
  scaleLin[3] = 0.000001;
  scaleExp[3] = 3.0;
  minScale[3] = 0.1;
  maxScale[3] = 10.0;
  scaleLin[4] = 0.00001;
  scaleExp[4] = 3.0;
  minScale[4] = 0.1;
  maxScale[4] = 10.0;
  scaleLin[5] = 0.001;
  scaleExp[5] = 0.0;
  minScale[5] = 0.1;
  maxScale[5] = 10.0;
  scaleLin[6] = 0.000001;
  scaleExp[6] = 3.0;
  minScale[6] = 0.1;
  maxScale[6] = 10.0;
  scaleLin[7] = 0.0001;
  scaleExp[7] = 2.0;
  minScale[7] = 0.1;
  maxScale[7] = 10.0;
  scaleLin[8] = 0.000001;
  scaleExp[8] = 3.0;
  minScale[8] = 0.1;
  maxScale[8] = 10.0;
  scaleLin[9] = 0.001;
  scaleExp[9] = 2.0;
  minScale[9] = 0.1;
  maxScale[9] = 10.0;
  for (i1 = 0; i1 < MAX_SCALES; i1++) {
    scale[i1] = 1.0;
  }
  for (i1 = 0; i1 < MAX_PARAMS; i1++) {
    harmonics[i1] = (double*)malloc(numHarmonics * sizeof(double));
    scaledHarmonics[i1] = (double*)malloc(numHarmonics * sizeof(double));
  }
  for (i1 = 0; i1 < MAX_PARAMS; i1++) {  
    for (i2 = 0; i2 < numHarmonics; i2++) {
      setHarmonic(i1, i2, 0);
    }
  }   
  for (i1 = 0; i1 < 2; i1++) {  
    for (i2 = 0; i2 < FILTER_LEN; i2++) {
      setFilter(i1, i2, 0);
    }
  }   
  synthSemaphore = new QSemaphore(poly);
  synthesisFlag = true;
  lfo = 0;
  lfoState = 1;
  lfoFreq = 12;
  lfoDepth = 0;
  dlfo = 0;
  editGroup = 0;
}

Model::~Model() {
  
  int i;

  for (i = 0; i < MAX_PARAMS; i++) {
    free(harmonics[i]);
  }    
  for (i = 0; i < poly; i++) {
    free(envelopes[i]);
  }    
  free(voices);
  free(synthSemaphore);
}

int Model::getRate() {

  return(rate);
}

void Model::setRate(int p_rate) {

  int i1;

  rate = p_rate;
  for (i1 = 0; i1 < poly; i1++) {
    voiceData[i1]->setRate(rate);
  }
}

int Model::getBufSize() {

  return(bufSize);
}

void Model::setBufSize(int p_bufSize) {

  bufSize = p_bufSize;
}

double Model::getPitchBend() {

  return(pitchBend);
}

void Model::setPitchBend(double p_pitchBend) {

  pitchBend = p_pitchBend;
}

bool Model::doSynthesis() {

  return(synthesisFlag);
}

double Model::getMax() {

  return(max);
}

void Model::setMax(double p_max) {

  max = p_max;
}

int Model::getPoly() {

  return(poly);
}

double Model::getMinScale(int p_index) {

  return(minScale[p_index]);
}

double Model::getMaxScale(int p_index) {

  return(maxScale[p_index]);
}

void Model::setPoly(int p_poly) {

  int i1, i2, oldPoly;

  oldPoly = poly;
  poly = p_poly;
  for (i1 = poly; i1 < oldPoly; i1++) {
    free(envelopes[i1]);
  }
  for (i1 = oldPoly; i1 < poly; i1++) {
    envelopes[i1] = (envelopeType *)malloc(numHarmonics * sizeof(double));
    for (i2 = 0; i2 < numHarmonics; i2++) {
      envelopes[i1][i2].val = 0;
      envelopes[i1][i2].state = 0;
      envelopes[i1][i2].lfo = 0;
      envelopes[i1][i2].lfo_state = 0;
    }
  }                  
  voices = (voiceType *)realloc(voices, poly * sizeof(voiceType));
  for (i1 = 0; i1 < poly; i1++) {
    voices[i1].note = 0;
    voices[i1].velocity = 0;
    voices[i1].gate = false;
    voices[i1].active = false;
    voices[i1].samples = 0;
  }
}

voiceType* Model::getVoices() {

  return(voices);
}

envelopeType* Model::getEnvelopes(int voiceIndex) {

  return(envelopes[voiceIndex]);
}

int Model::getNumHarmonics() {

  return(numHarmonics);
}

void Model::setNumHarmonics(int p_numHarmonics) {

  int i1, i2;
  
  numHarmonics = p_numHarmonics;
  for (i1 = 0; i1 < MAX_PARAMS; i1++) {
    harmonics[i1] = (double*)realloc(harmonics[i1], numHarmonics * sizeof(double));
  }
  for (i1 = 0; i1 < MAX_PARAMS; i1++) {
    for (i2 = 0; i2 < numHarmonics; i2++) {
      setHarmonic(i1, i2, 0);
    }
  }                
}

int Model::getHarmonicSplit(int p_group) {

  return(harmonicSplit[p_group]);
}

void Model::setHarmonicSplit(int p_group, int p_harmonicSplit) {

  harmonicSplit[p_group] = p_harmonicSplit;
}

double* Model::getHarmonics(int p_index) { 

  return(harmonics[p_index]);
}

double Model::getHarmonic(int p_index, int p_num) {
 
  if ((p_num >= 0) && (p_num < numHarmonics)) {
    return(harmonics[p_index][p_num]);
  } else {
    return(0);
  }
}

double Model::getScaledHarmonic(int p_index, int p_num) {
 
  if ((p_num >= 0) && (p_num < numHarmonics)) {
    return(scaledHarmonics[p_index][p_num]);
  } else {
    return(0);
  }
}

int Model::calcEditGroup(int p_index) {

  int e, i1;
  
  e = 0;
  for (i1 = 0; i1 < MAX_SPLIT; i1++) {
    if (p_index >= harmonicSplit[i1]) {
      e = i1;
    }
  }
  return(e);
}

void Model::setHarmonic(int p_index, int p_num, double p_value) {
 
  double val;
  int e, c;
  
  if ((p_num >= 0) && (p_num < numHarmonics)) {
    val = (p_value < 0) ? 0 : p_value;
    if (val > max) val = max;
    harmonics[p_index][p_num] = val;
    e = calcEditGroup(p_num);
    c = MAX_SCENES * MAX_SOURCES;
    switch (p_index) {
      case 0:
        scaledHarmonics[p_index][p_num] = (1.0 / sqrt((double)poly)) * scale[c * p_index + e] * scaleLin[p_index] * pow(10, val/20.0);
        break;
      case 1:
        scaledHarmonics[p_index][p_num] = (double)rate * scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index]);
        break;
      case 2:
        scaledHarmonics[p_index][p_num] = (double)STEP/((double)rate * (1e-5 + scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index])));
        break;
      case 3:
        scaledHarmonics[p_index][p_num] = exp(-0.693147 * (double)STEP/(1e-5 + (double)rate * scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index])));
        break;
      case 4:
        scaledHarmonics[p_index][p_num] = exp(-0.693147 * (double)STEP/(1e-5 + (double)rate * scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index])));
        break;
      case 5:
        scaledHarmonics[p_index][p_num] = scale[c * p_index + e] * 1e-6 * pow(10, val / 20.0);
        break;
      case 6:
        scaledHarmonics[p_index][p_num] = exp(-0.693147 * (double)STEP/(1e-5 + (double)rate * scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index])));
        break;
      case 7:
        scaledHarmonics[p_index][p_num] = scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index]);
        break;
      case 8:
        scaledHarmonics[p_index][p_num] = (double)rate * scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index]);
        break;
      case 9:
        scaledHarmonics[p_index][p_num] = 2.0 * scale[c * p_index + e] * scaleLin[p_index] * pow(val, scaleExp[p_index]) * (double)STEP / (double)rate;
        break;
    }    
  }
}

void Model::setGroupScale(int p_group, int p_index, int value) {

  int i1;
  
  if (p_index) {
    scale[MAX_SCENES * MAX_SOURCES * p_index + p_group] = pow(10.0, (double)value / 1000.0);
  } else {
    scale[MAX_SCENES * MAX_SOURCES * p_index + p_group] = pow(10.0, (double)value / 1000.0) - 0.1;
  }
//  printf("Scale: %d  %d  %f\n", p_group, p_index, scale[MAX_SCENES * MAX_SOURCES * p_index + p_group]);
  for (i1 = 0; i1 < numHarmonics; i1++) {
    setHarmonic(p_index, i1, harmonics[p_index][i1]);
  }
}

double Model::getLFO() {

  return(lfo);
}

void Model::lfoStep() {

  switch (lfoState) {
  case 1:
    if (lfo < lfoDepth) {
      lfo += dlfo;
      if (lfo > lfoDepth) {
        lfo = lfoDepth;
      }
    } else {
      lfoState = 2;
    }
    break;
  case 2:
    if (lfo > -lfoDepth) {
      lfo -= dlfo;
      if (lfo < -lfoDepth) {
        lfo = -lfoDepth;
      }
    } else {
      lfoState = 1;
    }
    break;
  }    
}
   
void Model::setLfoFreq(double value) {

  lfoFreq = value;
  dlfo = 2.0 * lfoDepth * lfoFreq * (double)STEP / rate;
}          

double Model::getLfoFreq() {

  return(lfoFreq);
}          

void Model::setLfoDepth(double value) {

  lfoDepth = value;
  dlfo = 2.0 * lfoDepth * lfoFreq * (double)STEP / rate;
}          

double Model::getLfoDepth() {

  return(lfoDepth);
}          

void Model::resetLFO() {

  lfo = 0;
  lfoState = 1;
}

int Model::getEditGroup() {

  return(editGroup);
}   
        
void Model::setEditGroup(int p_group) {     

  editGroup = p_group;
}

double Model::getMorphing() {

  return(morphing);          
} 

void Model::setMorphing(double value) {

  morphing = value;
}

double Model::getDelta(int index) {

  return(delta[index]);
}

void Model::setDelta(int index, double value) {

  delta[index] = value;
}

void Model::addControlObject(QString name, int objectType, int objectIndex, QTreeWidgetItem *item) {

  controlObject co;
  
  co.name = name;
  co.objectType = objectType;
  co.objectIndex = objectIndex;
  co.modDepth = 0;
  co.full = true;
  co.inverse = false;
  co.treeWidgetItem = item;
  controlObjectList.append(co);
}

int Model::hasMidiController(int type, unsigned char eventChannel,
                             unsigned int eventParam) {

  int l1;
  bool found;
  
  found = false;
  for (l1 = 0; l1 < midiControllerList.count(); l1++) {
    if ((midiControllerList.at(l1).eventChannel == eventChannel) 
        && (midiControllerList.at(l1).eventParam == eventParam)
        && (midiControllerList.at(l1).type == type)) {
      found = true;
      break;  
    }
  }
  return(found ? l1 : -1);                    
}                      
 
void Model::addMidiController(int type, unsigned char eventChannel,
                              unsigned int eventParam, QTreeWidgetItem *item) {
  midiController m;
  
  m.type = type;
  m.eventChannel = eventChannel;
  m.eventParam = eventParam;                        
  m.treeWidgetItem = item;    
  midiControllerList.append(m);                            
} 
    
bool Model::connectController(QTreeWidgetItem *midiControllerItem, 
                              QTreeWidgetItem *controlObjectItem) {

  int l1, l2, l3;
  bool notInList;
  midiController m;
  controlObject co1, co2;
  
  notInList = true;
  for (l1 = 0; l1 < midiControllerList.count(); l1++) {
    if (midiControllerList.at(l1).treeWidgetItem == midiControllerItem) {
      for (l2 = 0; l2 < controlObjectList.count(); l2++) {
        if (controlObjectList.at(l2).treeWidgetItem == controlObjectItem) {
          co1 = controlObjectList.at(l2);
          for (l3 = 0; l3 < midiControllerList.at(l1).connectedObjects.count(); l3++) {
            co2 = midiControllerList.at(l1).connectedObjects.at(l3);
            if ((co1.objectType == co2.objectType) && (co1.objectIndex == co2.objectIndex)) {
              notInList = false;
              break;
            }
          }
          if (notInList)  {
            m = midiControllerList.at(l1);
            m.connectedObjects.append(co1);
            midiControllerList.replace(l1, m);
          }  
          break;        
        }
      } 
      break;  
    }
  }
  return(notInList);
}

bool Model::connectController(QTreeWidgetItem *midiControllerItem,
                              int objectType, int objectIndex, int modDepth, bool full, bool inverse) { 

  int l1, l2, l3;
  bool notInList;
  midiController m;
  controlObject co1, co2;
  
  notInList = true;
  for (l1 = 0; l1 < midiControllerList.count(); l1++) {
    if (midiControllerList.at(l1).treeWidgetItem == midiControllerItem) {
      for (l2 = 0; l2 < controlObjectList.count(); l2++) {
        co1 = controlObjectList.at(l2);
        if ((co1.objectType == objectType) && (co1.objectIndex == objectIndex)) {
          break;
        } 
      }  
      for (l3 = 0; l3 < midiControllerList.at(l1).connectedObjects.count(); l3++) {
        co2 = midiControllerList.at(l1).connectedObjects.at(l3);
        if ((co2.objectType == objectType) && (co2.objectIndex == objectIndex)) {
          notInList = false;
          break;
        }
      }  
      if (notInList)  {
        m = midiControllerList.at(l1);
        co2.name = co1.name;
        co2.objectType = objectType;
        co2.objectIndex = objectIndex;
        co2.modDepth = modDepth;
        co2.full = full;
        co2.inverse = inverse;
        m.connectedObjects.append(co2);
        midiControllerList.replace(l1, m);
      } 
      break;  
    }
  }
  return(notInList);
}

bool Model::disconnectController(QTreeWidgetItem *midiControllerItem, QTreeWidgetItem *controlObjectItem) {
  
  int l1, l2;
  midiController m;
  bool success;
  
  success = false;
  for (l1 = 0; l1 < midiControllerList.count(); l1++) {
    if (midiControllerList.at(l1).treeWidgetItem == midiControllerItem) {
      for (l2 = 0; l2 < midiControllerList.at(l1).connectedObjects.count(); l2++) {
        if (midiControllerList.at(l1).connectedObjects.at(l2).treeWidgetItem == controlObjectItem) {
          m = midiControllerList.at(l1);
          m.connectedObjects.removeAt(l2);
          midiControllerList.replace(l1, m);
          success = true;
          break;        
        }
      } 
      break;  
    }
  }
  return(success);
}

midiController Model::getMidiController(int index) {

  return(midiControllerList.at(index));
}

controlObject Model::getControlObject(int index) {

  return(controlObjectList.at(index));
}

controlObject Model::getConnectedObject(int midiIndex, int controllerIndex) {

  return(midiControllerList.at(midiIndex).connectedObjects.at(controllerIndex));
}

void Model::setControllerItem(QTreeWidgetItem *midiControllerItem, 
                              QTreeWidgetItem *controlObjectItem, 
                              QTreeWidgetItem *connectedObjectItem) {

  int l1, l2, l3;
  midiController m;
  controlObject co1, co2;
  
  for (l1 = 0; l1 < midiControllerList.count(); l1++) {
    if (midiControllerList.at(l1).treeWidgetItem == midiControllerItem) {
      for (l2 = 0; l2 < controlObjectList.count(); l2++) {
        if (controlObjectList.at(l2).treeWidgetItem == controlObjectItem) {
          co1 = controlObjectList.at(l2);
          for (l3 = 0; l3 < midiControllerList.at(l1).connectedObjects.count(); l3++) {
            co2 = midiControllerList.at(l1).connectedObjects.at(l3);
            if ((co1.objectType == co2.objectType) && (co1.objectIndex == co2.objectIndex)) {
              m = midiControllerList.at(l1);
              co2.treeWidgetItem = connectedObjectItem;
              m.connectedObjects.replace(l3, co2);
              midiControllerList.replace(l1, m);
              break;
            }
          }
          break;            
        }
      } 
      break;  
    }
  }
}

void Model::setControllerItem(QTreeWidgetItem *midiControllerItem, 
                              int objectType, int objectIndex,  
                              QTreeWidgetItem *connectedObjectItem) {

  int l1, l2;
  midiController m;
  controlObject co;
  
  for (l1 = 0; l1 < midiControllerList.count(); l1++) {
    if (midiControllerList.at(l1).treeWidgetItem == midiControllerItem) {
      for (l2 = 0; l2 < midiControllerList.at(l1).connectedObjects.count(); l2++) {
        co = midiControllerList.at(l1).connectedObjects.at(l2);
        if ((co.objectType == objectType) && (co.objectIndex == objectIndex)) {
          m = midiControllerList.at(l1);
          co.treeWidgetItem = connectedObjectItem;
          m.connectedObjects.replace(l2, co);
          midiControllerList.replace(l1, m);
          break;
        }          
      } 
      break;  
    }
  }  
}

void Model::clearMidiControllerList() {

  midiControllerList.clear();
}      

void Model::clearControlObjectList() {

  controlObjectList.clear();
}
 
bool Model::isMidiControllerItem(QTreeWidgetItem *midiControllerItem) {

  int l1;
  
  for (l1 = 0; l1 < midiControllerList.count(); l1++) {
    if (midiControllerList.at(l1).treeWidgetItem == midiControllerItem) {
      return(true);
      break;
    }
  }    
  return(false);
}

QString Model::getControlObjectName(int objectType, int objectIndex) {

  int l1;
  controlObject co;
  QString qs;

  qs = "NOT FOUND";
  for (l1 = 0; l1 < controlObjectList.count(); l1++) {
    co = controlObjectList.at(l1);
    if ((co.objectType == objectType) && (co.objectIndex == objectIndex)) {
      qs = co.name;
      break;      
    }
  }    
  return(qs);
}

int Model::midiControllerCount() {

  return(midiControllerList.count());
}

int Model::connectedObjectCount(int midiControllerIndex) {

  return(midiControllerList.at(midiControllerIndex).connectedObjects.count());
}
             
double *Model::getFilter(int p_index) {

  return(filter[p_index]);
}            

double Model::getFilter(int p_index, int p_num) {

  return(filter[p_index][p_num]);
}

void Model::setFilter(int p_index, int p_num, double p_value) {

  if (p_value > max) p_value = max;
  filter[p_index][p_num] = (p_value < 0) ? 0 : p_value;
}

double Model::getFilterCutoff(int p_index) {

  return(filterCutoff[p_index]);
}

double Model::getFilterSpacing(int p_index) {

  return(filterSpacing[p_index]);
}

double Model::getFilterKeyTracking(int p_index) {

  return(filterKeyTracking[p_index]);
}

double Model::getFilterDepth(int p_index) {

  return(filterDepth[p_index]);
}

void Model::setFilterCutoff(int p_index, double p_value) {

  filterCutoff[p_index] = p_value;
}

void Model::setFilterSpacing(int p_index, double p_value) {

  filterSpacing[p_index] = p_value;
}

void Model::setFilterKeyTracking(int p_index, double p_value) {

  filterKeyTracking[p_index] = p_value;
}

void Model::setFilterDepth(int p_index, double p_value) {

  filterDepth[p_index] = p_value;
}

void Model::setFilterMixMorph(int spectrumIndex, double value) {

  filterMixMorph[spectrumIndex] = value;
}

void Model::setFilterMixMode(int spectrumIndex, double value) {

  filterMixMode[spectrumIndex] = value;
}

void Model::setFilterMixEnvAmount(int spectrumIndex, double value) {

  filterMixEnvAmount[spectrumIndex] = value;
}

void Model::setFilterMixDelay(int spectrumIndex, double value) {

  filterMixDelay[spectrumIndex] = value;
}

void Model::setFilterMixAttack(int spectrumIndex, double value) {

  filterMixAttack[spectrumIndex] = value;
}

void Model::setFilterMixDecay(int spectrumIndex, double value) {

  filterMixDecay[spectrumIndex] = value;
}

void Model::setFilterMixLfo(int spectrumIndex, bool on) {

  filterMixLfo[spectrumIndex] = on;
}

void Model::setFilterMixRetrigger(int spectrumIndex, bool on) {

  filterMixRetrigger[spectrumIndex] = on;
}

void Model::setFilterMixHold(int spectrumIndex, bool on) {

  filterMixHold[spectrumIndex] = on;
}

double Model::getFilterMixMorph(int spectrumIndex) {

  return(filterMixMorph[spectrumIndex]);
}

double Model::getFilterMixMode(int spectrumIndex) {

  return(filterMixMode[spectrumIndex]);
}

double Model::getFilterMixEnvAmount(int spectrumIndex) {

  return(filterMixEnvAmount[spectrumIndex]);
}

double Model::getFilterMixDelay(int spectrumIndex) {

  return(filterMixDelay[spectrumIndex]);
}

double Model::getFilterMixAttack(int spectrumIndex) {

  return(filterMixAttack[spectrumIndex]);
}

double Model::getFilterMixDecay(int spectrumIndex) {

  return(filterMixDecay[spectrumIndex]);
}

bool Model::getFilterMixLfo(int spectrumIndex) {

  return(filterMixLfo[spectrumIndex]);
}

bool Model::getFilterMixRetrigger(int spectrumIndex) {

  return(filterMixRetrigger[spectrumIndex]);
}

bool Model::getFilterMixHold(int spectrumIndex) {

  return(filterMixHold[spectrumIndex]);
}

void Model::setFilterModEnvelope(int filterIndex, modulationEnvelope p_Envelope) {

  filterModEnvelope[filterIndex] = p_Envelope;
//  printf("%d: Delay: %f  Attack: %f  Decay 1: %f  Decay 2: %f  Level: %f  Release: %f  Amount: %f\n",
//         filterIndex, filterModEnvelope[filterIndex].delay, filterModEnvelope[filterIndex].attack, filterModEnvelope[filterIndex].decay1, filterModEnvelope[filterIndex].decay2, 
//         filterModEnvelope[filterIndex].level, filterModEnvelope[filterIndex].release, filterModEnvelope[filterIndex].amount);  
}

void Model::setFilterModLfo(int filterIndex, modulationLfo p_Lfo) {

  filterModLfo[filterIndex] = p_Lfo;
//  printf("LFO %d: Delay: %f  Frequency: %f  Depth: %f\n",
//         filterIndex, filterModLfo[filterIndex].delay, filterModLfo[filterIndex].freq, filterModLfo[filterIndex].depth);
}

void Model::setFilterModSpacingLfo(int filterIndex, modulationLfo p_Lfo) {

  filterModSpacingLfo[filterIndex] = p_Lfo;
//  printf("Spacing LFO %d: Delay: %f  Frequency: %f  Depth: %f\n",
//         filterIndex, filterModSpacingLfo[filterIndex].delay, filterModSpacingLfo[filterIndex].freq, filterModSpacingLfo[filterIndex].depth);
}

modulationEnvelope Model::getFilterModEnvelope(int filterIndex) {

  return(filterModEnvelope[filterIndex]);
}

modulationLfo Model::getFilterModLfo(int filterIndex) {

  return(filterModLfo[filterIndex]);
}

modulationLfo Model::getFilterModSpacingLfo(int filterIndex) {

  return(filterModSpacingLfo[filterIndex]);
}

void Model::setEvenOdd(int index, double value) {

  evenOdd[index] = value;
//  printf("evenOdd %d = %f\n", index, evenOdd[index]);
}

double Model::getEvenOdd(int index) {

  return(evenOdd[index]);
}

void Model::setMasterVolume(double value) {

  masterVolume = value;
}
        
double Model::getMasterVolume() {

  return(masterVolume);
}

void Model::setNoiseBandwidth(double value) {

  noiseBandwidth = value;
}

double Model::getNoiseBandwidth() {

  return(noiseBandwidth);       
}

void Model::setControllerModulation(QTreeWidgetItem *midiControllerItem, QTreeWidgetItem *controlObjectItem, int modDepth, bool full, bool inverse) {

  int i1, i2;
  controlObject co;
  midiController m;

  for (i1 = 0; i1 < midiControllerList.count(); i1++) {
    if (midiControllerList.at(i1).treeWidgetItem == midiControllerItem) {
      for (i2 = 0; i2 < midiControllerList.at(i1).connectedObjects.count(); i2++) {
        co = midiControllerList.at(i1).connectedObjects.at(i2);
        if (co.treeWidgetItem == controlObjectItem) {
          co.modDepth = modDepth;
          co.full = full;
          co.inverse = inverse;
          m = midiControllerList.at(i1);
          m.connectedObjects.replace(i2, co);
          midiControllerList.replace(i1, m);
//          printf("setControllerModulation -> mList: %d  cList: %d  v: %d  f: %d  i: %d\n", i1, i2, modDepth, full, inverse);
          break;
        }
      } 
      break;  
    }
  }
}

controlObject Model::getControlObject(QTreeWidgetItem *midiControllerItem, QTreeWidgetItem *controlObjectItem, bool& success) {

  int i1, i2;
  controlObject co;
  midiController m;

  success = false;
  for (i1 = 0; i1 < midiControllerList.count(); i1++) {
    if (midiControllerList.at(i1).treeWidgetItem == midiControllerItem) {
      for (i2 = 0; i2 < midiControllerList.at(i1).connectedObjects.count(); i2++) {
        co = midiControllerList.at(i1).connectedObjects.at(i2);
        if (co.treeWidgetItem == controlObjectItem) {
          success = true;
          break;
        }
      } 
      break;  
    }
  }
  return(co);
}

void Model::setCompressor(int index, double value) {

//  printf("\n setCompressor %d  %f\n", index, value);
  switch (index) {
    case 0:
      linearLevel = 0.008 * pow(10, value / 500.0);
      break;
    case 1:
      transitionWidth = 0.03 * pow(10, value / 500.0);
      break;
    case 2:
      compression = 0.005 * pow(10, value / 500.0);
      break;    
    case 3:
      antialiasing = 1.0 + (1000.0 - value) / 66.6666667;
      break;    
  }
/*
  double x0, x1, a, b, c, d;
  x0 = (1.0 / sqrt((double)poly)) * (double)MAX_LEVEL * linearLevel;
  x1 = x0 + (1.0 / sqrt((double)poly)) * (double)MAX_LEVEL * transitionWidth;
  d = compression;
  c = x0*(1.0 - d) + (x0*x1)/(x0-x1) - x0*x0/(2.0 * (x0-x1));
  b = 1.0/(2.0 * (x0-x1));
  a = d - x1/(x0-x1);
  printf("x0 %f  x1 %f  a %f  b %f  c %f  d %f\n", x0, x1, a, b * 1e6, c, d);
*/
}

void Model::getCompressorCoeff(double& x0, double& x1, double& a, double& b, double& c, double& d, double& aa) {

  x0 = (1.0 / (double)poly) * (double)MAX_LEVEL * linearLevel;
  x1 = x0 + (1.0 / (double)poly) * (double)MAX_LEVEL * transitionWidth;
  d = compression;
  c = x0*(1.0 - d) + (x0*x1)/(x0-x1) - x0*x0/(2.0 * (x0-x1));
  b = 1.0/(2.0 * (x0-x1));
  a = d - x1/(x0-x1);
  aa = antialiasing;
}

void Model::setVerbose(bool on) {

  verbose = on;
}

bool Model::getVerbose() {

  return(verbose);
}

void Model::setEditRange(int start, int end) {

  editStart = start;
  editEnd = end;
}

void Model::getEditRange(int& start, int& end) {

  start = editStart;
  end = editEnd;
}

VoiceData* Model::getVoiceData(int index) {

  return(voiceData[index]);
}

VoiceData* Model::currentVoiceData() {

  return(voiceData[currentVoiceDataIndex]);
}

void Model::setCurrentVoiceDataIndex(int index) {

  currentVoiceDataIndex = index;
}
